//
//	Xanthus
//	Copyright (c) 1992, 1993, 1994 Xanthus International AB.  
//	All rights reserved. 
//

#import "app.h"

int Debug = 1;

@implementation APIDemo

- reportError: (const char * ) format, ...;
{
	va_list a; 
	char alert[2048];

	va_start(a, format);
	vsprintf(alert, format, a);
	va_end(a);
	
	NXRunAlertPanel("Error", alert, "OK", NULL, NULL);
	
	return self;
}

- launch:sender;
{
	BOOL x;
	
	if(theServerProxy != nil) return theServerProxy;
	
     // Check if running
    
 	theServerProxy = [NXConnection connectToName: OPENWRITE_SERVER_NAME];
	if(theServerProxy != nil) goto exit;
		
    // Check if OW launched
	
	x = [[Application workspace]
			launchApplication:OPENWRITE];

	sleep(5);	// Hack: sleep 5 seconds so that
			// OpwnWrite is launched properly
	
	if(x == NO) {
	
		x = [[Application workspace]
				launchApplication:OPENWRITE_PRO];
	
		sleep(5);	// Hack: sleep 5 seconds so that
				// OpwnWrite is launched properly
	}
	
    exit:
 	
	theServerProxy = [NXConnection connectToName: OPENWRITE_SERVER_NAME];
	
	if(theServerProxy == nil) {
		[self reportError: 
			"Could not connect to %s!", 
						OPENWRITE_SERVER_NAME];
		return nil;
	}

     	[[theServerProxy connectionForProxy]
		 registerForInvalidationNotification:self];
		 
    // allow server to talk back to us == sender is invalid protocol
    
    	[[theServerProxy connectionForProxy] runFromAppKit];
	
   	return self;
}

- load:sender
{
	if(theLoadFlag) return self;
	
	theLoadFlag = [theServerProxy loadAPI];
	
	if(theLoadFlag == 0) {
		[self reportError: 
			"Could not load API module into %s!", OPENWRITE];
		return nil;
	}
    	return self;
}

- terminate:sender
{
	NX_DURING
		[theServerProxy API_terminate];
	NX_HANDLER
		;
	NX_ENDHANDLER
	
	[self reset];
    	return self;
}

- reset;
{
	theServerProxy = nil;
	theDocumentProxy = nil;
	theLoadFlag = 0;
	return self;
}

- senderIsInvalid:sender;
{
	if(Debug)
		NXLogError("senderIsInvalid '%s'.", [sender name]);
		
	if([theServerProxy connectionForProxy] == sender) {
	
		if(Debug) NXLogError("Server is invalid.");
	
		[self reset];
	}
	
	return self;
}

- createWithoutWindow:sender;
{
	if(theDocumentProxy) return self;
	
	theDocumentProxy = [theServerProxy API_newDocumentSilently];
 
 	if(theDocumentProxy == 0) {
		[self reportError: 
			"Could not create a new %s document!", OPENWRITE];
		return nil;
	}
   	return self;
}

- create:sender
{
	if(theDocumentProxy) return self;
	
	theDocumentProxy = [theServerProxy API_newDocument];
 
 	if(theDocumentProxy == 0) {
		[self reportError: 
			"Could not create a new %s document!", OPENWRITE];
		return nil;
	}
   	return self;
}

- document:sender
{
	int aTag = [sender selectedTag];
	int code = 0;
	
	if(theDocumentProxy == nil) return self;
	
	NX_DURING
	
		switch(aTag) {
		
		case 1 :
			[theDocumentProxy API_hide];
			break;
		
		case 2 :
			[theDocumentProxy API_unhide];
			break;
		
		case 3 :
			[theDocumentProxy API_close];
			theDocumentProxy = nil;
			break;
		
		case 4 :
			[theDocumentProxy API_saveAs: "/tmp/temp.xow"];
			break;
		}
		
	NX_HANDLER
		code = NXLocalHandler.code;
	NX_ENDHANDLER
	
 	if(code != 0) {
		[self reset];
		[self reportError: 
			"DO Exception %d when sending to doc.", code];
		return nil;
	}
	
	
   	return self;
}

//
//
//

- windowDidUpdate:sender;
{
	[theButton1 setEnabled: theServerProxy == nil];
	
	[theButton2 setEnabled: theServerProxy != nil &&
					 theLoadFlag == 0];
	
	[theButton3 setEnabled: theServerProxy != nil &&
					 theDocumentProxy == nil];
					 
	[theButton3b setEnabled: theServerProxy != nil &&
					 theDocumentProxy == nil];
					 
	[theButton4 setEnabled: theServerProxy != nil &&
					 theDocumentProxy != nil];
	
	[theButton5 setEnabled: theServerProxy != nil];
	
	return self;
}

- appWillInit:sender;
{
	[NXApp setAutoupdate:YES];
	return self;
}

@end
