//
//	Xanthus
//	Copyright (c) 1992, 1993, 1994 Xanthus International AB.  
//	All rights reserved. 
//

//
// The name of the two wordprocessors
//

#define OPENWRITE 		"OpenWrite"
#define OPENWRITE_PRO 		"OpenWritePro"


//
// This is the name of the Distributed Objects server
// Connect to this with:
// theServerProxy = [NXConnection connectToName: OPENWRITE_SERVER_NAME];
//

#define OPENWRITE_SERVER_NAME 	"OpenWrite API Server"


//
// Top level API
// Before any other API methods can be used, the API must be loaded
// with this method
//

@protocol OpenWriteAPIInit

- (int)loadAPI;					// returns 1 = success 
						// or 0 = failure

@end


//
// This protocol can only be used after the 
// API has been loaded with loadAPI
// Messages should be sent to the server proxy
//

@protocol OpenWriteAPIRoot

- API_newDocument;				// Returns proxy to document
- API_openDocument: (const char *)path;		// Returns proxy to document
- API_openDocumentReadOnly: (const char *)path;	// Returns proxy to document

// New in OpenWrite 1.1
- API_openDocumentSilently: (const char *)path;	// Returns proxy to document
- API_newDocumentSilently;			// Returns proxy to document

//
- (void)API_terminate;				//
- (void)API_hide;				//
- (void)API_unhide;				//
- (void)API_debug:(int)flag;			//

@end


//
// The basic document API
// Messages to be sent to document proxys
//

@protocol OpenWriteAPIDocument

- (int)API_saveTo: (const char *)path;
- (int)API_save;
- (int)API_saveAs: (const char *)path;
- (const char *)API_path;

- (void)API_setDocEdited: (int)flag;
- (int)API_isDocEdited;

- (void)API_hide;
- (void)API_unhide;

- (void)API_close;

@end


//
// The merge document API
// Messages to be sent to document proxys
//

@protocol OpenWriteAPIDocumentMerge

- (void)API_setMergeSource: (const char *)path;
- API_mergeFrom: (int)start to: (int)end;	// Returns proxy to document
- API_merge;					// Returns proxy to document

@end




//
// The print document API
// Messages to be sent to document proxys
//

@protocol OpenWriteAPIDocumentPrint


- (int)API_runPrintPanel;		//
- (int)API_print;			//

// New in OpenWrite 1.1
- (oneway void)API_printAsynchronously;			//

// New in OpenWrite 1.1
- (oneway void)API_runPrintPanelAsynchronously;		//


- (PrintInfo *)API_getPrintInfo;	// Returns proxy to PrintInfo *
					// Use any PrintInfo method
					// to set print features
					
// New in OpenWrite 1.1
- (int)API_saveToPostScript: (const char *)psFile;

@end


//
// The selection document API
// Messages to be sent to document proxys
//

@protocol OpenWriteAPIDocumentSelection

- (unsigned int)API_characterCount;
- (void)API_setSelectionStart: (int)from length: (int)length;
- (void)API_getSelectionStart: (int *)from length: (int *)length;
- (const char *)API_copySelectionAsASCII;
- (void)API_replaceSelectionWithASCII: (const char *)buf;

@end


//
// The Display API
// Messages to be sent to document proxys
// Added to OpenWrite 1.1

@protocol OpenWriteAPIDocumentDisplay

- (void)API_disableDisplay;
- (void)API_reenableDisplay;
- (void)API_display;

@end

