//
//	Xanthus
//	Copyright (c) 1992, 1993, 1994 Xanthus International AB.  
//	All rights reserved. 
//

#import "app.h"

int Debug = 1;

@implementation APIDemo

- reportError: (const char * ) format, ...;
{
	va_list a; 
	char alert[2048];

	va_start(a, format);
	vsprintf(alert, format, a);
	va_end(a);
	
	NXRunAlertPanel("Error", alert, "OK", NULL, NULL);
	
	return self;
}

- launch:sender;
{
	BOOL x;
	
	if(theServerProxy != nil) return theServerProxy;

    // Check if running
    
 	theServerProxy = [NXConnection connectToName: OPENWRITE_SERVER_NAME];
	if(theServerProxy != nil) goto exit;
		
    // Check if OW launched
	
	x = [[Application workspace]
			launchApplication:OPENWRITE];

	sleep(5);	// Hack: sleep 5 seconds so that
			// OpenWrite is launched properly
	
	if(x == NO) {
	
		x = [[Application workspace]
				launchApplication:OPENWRITE_PRO];
	
		sleep(5);	// Hack: sleep 5 seconds so that
				// OpwnWrite is launched properly
	}
	
    exit:
    
 	theServerProxy = [NXConnection connectToName: OPENWRITE_SERVER_NAME];
	
	if(theServerProxy == nil) {
		[self reportError: 
			"Could not connect to %s!", 
						OPENWRITE_SERVER_NAME];
		return nil;
	}

     	[[theServerProxy connectionForProxy]
		 registerForInvalidationNotification:self];
		 
    // allow server to talk back to us == sender is invalid protocol
    
    	[[theServerProxy connectionForProxy] runFromAppKit];
	
     // bring my window to front!
    
	[NXApp unhide:nil];
	
  	return self;
}

- load:sender
{
	if(theLoadFlag) return self;
	
	theLoadFlag = [theServerProxy loadAPI];
	
	if(theLoadFlag == 0) {
		[self reportError: 
			"Could not load API module into %s!", OPENWRITE];
		return nil;
	}
    	return self;
}

- terminate:sender
{
	NX_DURING
		[theServerProxy API_terminate];
	NX_HANDLER
		;
	NX_ENDHANDLER
	
	[self reset];
    	return self;
}

- reset;
{
	theMergedProxy = nil;
	theServerProxy = nil;
	theDocumentProxy = nil;
	theLoadFlag = 0;
	return self;
}

- senderIsInvalid:sender;
{
	if(Debug)
		NXLogError("senderIsInvalid '%s'.", [sender name]);
		
	if([theServerProxy connectionForProxy] == sender) {
	
		if(Debug) NXLogError("Server is invalid.");
	
		[self reset];
	}
	
	if([theDocumentProxy connectionForProxy] == sender) {	// never hap

		if(Debug) NXLogError("Doc is invalid.");

		theDocumentProxy = nil;
	}
	
	if([theMergedProxy connectionForProxy] == sender) {	// never hap

		if(Debug) NXLogError("Merged Doc is invalid.");

		theMergedProxy = nil;
	}
	
	return self;
}

- open:sender
{
	char path[2048];
	char appdir[2048];
	int code = 0;
	
	if(theDocumentProxy) return self;
	
	[self getAppDirectory: appdir];
	sprintf(path, "%s/Document.xow", appdir);   
	
    	NX_DURING
		theDocumentProxy = [theServerProxy API_openDocument:path];
 	NX_HANDLER
		code = NXLocalHandler.code;
	NX_ENDHANDLER
	
 	if(code != 0) {
		[self reportError: 
			"DO Exception %d for 'API_openDocument:'", code];
		return nil;
	}
 
 	if(theDocumentProxy == 0) {
		[self reportError: 
			"Could not open document %s!", path];
		return nil;
	}

    	[[theDocumentProxy connectionForProxy]
		 registerForInvalidationNotification:self];
    
   	return self;
}

- merge:sender
{
	static char path[2048];
	static char appdir[2048];
	int code = 0;
	
	if(theMergedProxy != nil) return self;
	if(theDocumentProxy == nil) return self;
	
	[self getAppDirectory: appdir];
	sprintf(path, "%s/Data.xom", appdir);
	
    // set source
    
    	NX_DURING
		[theDocumentProxy API_setMergeSource:path];
 	NX_HANDLER
		code = NXLocalHandler.code;
	NX_ENDHANDLER
	
 	if(code != 0) {
		[self reportError: 
			"DO Exception %d for 'API_setMergeSource:'", code];
		[self reset];
		return nil;
	}

    // merge
    
     	NX_DURING
		theMergedProxy = [theDocumentProxy API_merge];
 	NX_HANDLER
		code = NXLocalHandler.code;
	NX_ENDHANDLER
	
  	if(code != 0) {
		[self reportError: 
			"DO Exception %d for 'API_merge'", code];
		return nil;
	}

	if(theMergedProxy == 0) {
		[self reportError: 
			"Could not merge a new %s document!", OPENWRITE];
		return nil;
	}
  
     	[[theMergedProxy connectionForProxy]
		 registerForInvalidationNotification:self];

 	return self;
}

- print:sender
{	
	int code = 0;
	
	if(theMergedProxy == nil) return nil;
	
     	NX_DURING
		[theMergedProxy API_runPrintPanelAsynchronously];
 	NX_HANDLER
		code = NXLocalHandler.code;
	NX_ENDHANDLER

   	if(code != 0) {
		[self reset];
		[self reportError: 
			"DO Exception %d for "
			"'API_runPrintPanelAsynchronously'", code];
		return nil;
	}

   	return self;
}

- close:sender
{	
	int code = 0;
	
	if(theMergedProxy == nil) return nil;
	
     	NX_DURING
		[theMergedProxy API_close];
 	NX_HANDLER
		code = NXLocalHandler.code;
	NX_ENDHANDLER

	theMergedProxy = nil;
	
   	if(code != 0) {
		[self reset];
		[self reportError: 
			"DO Exception %d for 'API_close'", code];
		return nil;
	}

   	return self;
}

//
//
//

- windowDidUpdate:sender;
{
	[theButton1 setEnabled: theServerProxy == nil];
	
	[theButton2 setEnabled: theServerProxy != nil &&
					 theLoadFlag == 0];
	
	[theButton3 setEnabled: theServerProxy != nil &&
					 theDocumentProxy == nil &&
					 theLoadFlag];
					 
	[theButton4 setEnabled: theMergedProxy == nil && 
				theDocumentProxy != nil];
					 
	[theButton5 setEnabled: theMergedProxy != nil];
	
	[theButton6 setEnabled: theMergedProxy != nil];
	
	return self;
}

- appWillInit:sender;
{
	[NXApp setAutoupdate:YES];
	return self;
}

- getAppDirectory: (STR)appDirectory;
{
	FILE *process;
	char command[256];
	char *suffix;
	static int flag = 0;
	static char _appDir[2048];
	
	if(flag) {
		strcpy(appDirectory, _appDir);
		return self;
	}
	
	
	strcpy(appDirectory, NXArgv[0]);
	
	if (appDirectory[0] == '/') { 		// if absolute path 
	    if (suffix = rindex(appDirectory,'/'))
		*suffix  = '\0'; 		// remove executable name
	} else {
	    sprintf(command,"which '%s'\n",NXArgv[0]);
	    process=popen(command,"r");
	    fscanf(process,"%s",appDirectory);
	    pclose(process);
	    if (suffix = rindex(appDirectory,'/'))
		*suffix  = '\0'; 		// remove executable name
	    chdir(appDirectory);
	    getwd(appDirectory);
	} 
	
	strcpy(_appDir, appDirectory);
	flag = 1;
	
	return self;
}

@end
