//
//	Xanthus
//	Copyright (c) 1992, 1993, 1994 Xanthus International AB.  
//	All rights reserved. 
//

#import "app.h"

int Debug = 1;

@implementation APIDemo

- reportError: (const char * ) format, ...;
{
	va_list a; 
	char alert[2048];

	va_start(a, format);
	vsprintf(alert, format, a);
	va_end(a);
	
	NXRunAlertPanel("Error", alert, "OK", NULL, NULL);
	
	return self;
}

- launch:sender;
{
	BOOL x;
	
	if(theServerProxy != nil) return theServerProxy;

    // Check if running
    
 	theServerProxy = [NXConnection connectToName: OPENWRITE_SERVER_NAME];
	if(theServerProxy != nil) goto exit;
		
    // Check if OW launched
	
	x = [[Application workspace]
			launchApplication:OPENWRITE];

	sleep(5);	// Hack: sleep 5 seconds so that
			// OpenWrite is launched properly
	
	if(x == NO) {
	
		x = [[Application workspace]
				launchApplication:OPENWRITE_PRO];
	
		sleep(5);	// Hack: sleep 5 seconds so that
				// OpwnWrite is launched properly
	}
	
	
    exit:
    
 	theServerProxy = [NXConnection connectToName: OPENWRITE_SERVER_NAME];
	
	if(theServerProxy == nil) {
		[self reportError: 
			"Could not connect to %s!", 
						OPENWRITE_SERVER_NAME];
		return nil;
	}

     	[[theServerProxy connectionForProxy]
		 registerForInvalidationNotification:self];
		 
    // allow server to talk back to us == sender is invalid protocol
    
    	[[theServerProxy connectionForProxy] runFromAppKit];
	
     // bring my window to front!
    
	[NXApp unhide:nil];
	
  	return self;
}

- load:sender
{
	if(theLoadFlag) return self;
	
	theLoadFlag = [theServerProxy loadAPI];
	
	if(theLoadFlag == 0) {
		[self reportError: 
			"Could not load API module into %s!", OPENWRITE];
		return nil;
	}
    	return self;
}

- reset;
{
	theServerProxy = nil;
	theDocumentProxy = nil;
	theLoadFlag = 0;
	return self;
}

- terminate:sender;
{
	NX_DURING
		[theServerProxy API_terminate];
	NX_HANDLER
		;
	NX_ENDHANDLER
	
	[self reset];
    	return self;
}

- open:sender;
{
	char path[2048];
	char appdir[2048];
	int code = 0;
	
	if(theDocumentProxy) return self;
	
	[self getAppDirectory: appdir];
	sprintf(path, "%s/Document.xow", appdir);   
	
    	NX_DURING
		theDocumentProxy = [theServerProxy API_openDocument:path];
 	NX_HANDLER
		code = NXLocalHandler.code;
	NX_ENDHANDLER
	
 	if(code != 0) {
		[self reportError: 
			"DO Exception %d for 'API_openDocument:'", code];
		return nil;
	}
 
 	if(theDocumentProxy == 0) {
		[self reportError: 
			"Could not open document %s!", path];
		return nil;
	}
   	return self;
}

- senderIsInvalid:sender;
{
	if(Debug)
		NXLogError("senderIsInvalid '%s'.", [sender name]);
		
	if([theServerProxy connectionForProxy] == sender) {
	
		if(Debug) NXLogError("Server is invalid.");
	
		[self reset];
	}
	
	return self;
}

- getCharacterCount:sender;
{
	int code = 0;
	int c = 0;
	
	if(theDocumentProxy == nil) return self;
	
    	NX_DURING
		c = [theDocumentProxy API_characterCount];
 	NX_HANDLER
		code = NXLocalHandler.code;
	NX_ENDHANDLER
	
 	if(code != 0) {
		[self reset];
		[self reportError: 
			"DO Exception %d for 'API_characterCount'", code];
		return nil;
	}
	
	[theCharCount setIntValue: c];
 
   	return self;
}

- getSelection:sender;
{
	int code = 0;
	int s = 0;
	int l = 0;
	
	if(theDocumentProxy == nil) return self;
	
    	NX_DURING
		[theDocumentProxy API_getSelectionStart: &s length: &l];
 	NX_HANDLER
		code = NXLocalHandler.code;
	NX_ENDHANDLER
	
 	if(code != 0) {
		[self reset];
		[self reportError: 
			"DO Exception %d for 'API_getSelectionStart:length:'",
					 code];
		return nil;
	}
	
	[theGetSelStart setIntValue: s];
	[theGetSelLen setIntValue: l];
 
   	return self;
}

- setSelection:sender
{
	int code = 0;
	int s = [theSetSelStart intValue];
	int l = [theSetSelLen intValue];
	
	if(theDocumentProxy == nil) return self;
	
    	NX_DURING
		[theDocumentProxy API_setSelectionStart: s length: l];
 	NX_HANDLER
		code = NXLocalHandler.code;
	NX_ENDHANDLER
	
 	if(code != 0) {
		[self reset];
		[self reportError: 
			"DO Exception %d for 'API_setSelectionStart:length:'",
					 code];
		return nil;
	}
	
   	return self;
}

- getASCII:sender
{
	int code = 0;
	const char *buf;
	
	if(theDocumentProxy == nil) return self;
	
    	NX_DURING
		buf = [theDocumentProxy API_copySelectionAsASCII];
 	NX_HANDLER
		code = NXLocalHandler.code;
	NX_ENDHANDLER
	
 	if(code != 0) {
		[self reset];
		[self reportError: 
			"DO Exception %d for 'API_copySelectionAsASCII'",
					 code];
		return nil;
	}
	
	[theGetASCII setStringValue: buf];
	
   	return self;
}

- setASCII:sender
{
	int code = 0;
	const char *buf = [theSetASCII stringValue];
	
	if(theDocumentProxy == nil) return self;
	
    	NX_DURING
		[theDocumentProxy API_replaceSelectionWithASCII:buf];
 	NX_HANDLER
		code = NXLocalHandler.code;
	NX_ENDHANDLER
	
 	if(code != 0) {
		[self reset];
		[self reportError: 
			"DO Exception %d for 'API_replaceSelectionWithASCII:'",
					 code];
		return nil;
	}
	
   	return self;
}

- close:sender
{	
	int code = 0;
	
	if(theDocumentProxy == nil) return nil;
	
     	NX_DURING
		[theDocumentProxy API_close];
 	NX_HANDLER
		code = NXLocalHandler.code;
	NX_ENDHANDLER

	theDocumentProxy = nil;
	
   	if(code != 0) {
		[self reset];
		[self reportError: 
			"DO Exception %d for 'API_close'", code];
		return nil;
	}

   	return self;
}

//
//
//

- windowDidUpdate:sender;
{
	const char *ascii = [theSetASCII stringValue];
	
	[theButton1 setEnabled: theServerProxy == nil];		// launch
	
	[theButton2 setEnabled: theServerProxy != nil &&	// load API
					 theLoadFlag == 0];
	
	[theButton3 setEnabled: theServerProxy != nil &&	// open doc
					 theDocumentProxy == nil &&
					 theLoadFlag];
					 
	[theButton4 setEnabled: theDocumentProxy != nil];	// get chars 
					 
	[theButton5 setEnabled: theDocumentProxy != nil];	// get sel
		
	[theButton6 setEnabled: theDocumentProxy != nil];	// set sel
	
	[theButton7 setEnabled: theDocumentProxy != nil];	// get ascii
	
	[theButton8 setEnabled: theDocumentProxy != nil &&
				ascii &&
				strlen(ascii) > 0];	// set ascii
	
	[theButton9 setEnabled: theDocumentProxy != nil];	// close
	
	return self;
}

- appWillInit:sender;
{
	[NXApp setAutoupdate:YES];
	return self;
}

- getAppDirectory: (STR)appDirectory;
{
	FILE *process;
	char command[256];
	char *suffix;
	static int flag = 0;
	static char _appDir[2048];
	
	if(flag) {
		strcpy(appDirectory, _appDir);
		return self;
	}
	
	
	strcpy(appDirectory, NXArgv[0]);
	
	if (appDirectory[0] == '/') { 		// if absolute path 
	    if (suffix = rindex(appDirectory,'/'))
		*suffix  = '\0'; 		// remove executable name
	} else {
	    sprintf(command,"which '%s'\n",NXArgv[0]);
	    process=popen(command,"r");
	    fscanf(process,"%s",appDirectory);
	    pclose(process);
	    if (suffix = rindex(appDirectory,'/'))
		*suffix  = '\0'; 		// remove executable name
	    chdir(appDirectory);
	    getwd(appDirectory);
	} 
	
	strcpy(_appDir, appDirectory);
	flag = 1;
	
	return self;
}

@end
