//
//	Copyright (c) 1995 Bjorn Backlund.  
//	All rights reserved. 
//

#import "app.h"


@implementation Cosmo_Doc (Closing)

- free;
{
    // Free the polylines
    
	[thePolylines freeObjects];
	[thePolylines free];
	thePolylines = nil;
	
    // Free the path
    
    	if(thePath != NULL) {
	
		NX_FREE(thePath);
		thePath = NULL;
	}
	
    // Free the window and the view
    
    	[theWindow free];
	theWindow = nil;
	
     // Free the OOE client path
    
    	if(theOOEClientPath != NULL) {
	
		NX_FREE(theOOEClientPath);
		theOOEClientPath = NULL;
	}
	
   // Call the inherited version
    
	return [super free];
}

//
//
//

- (int)confirmClose;
{
	int a;
	
    // Use the builtin NEXTSTEP confirm panel to let user
    // decide what to do
	
	a = NXRunAlertPanel("Close Document",
	
		"The document contains unsaved changes.",
		
		"Close Anyway",
		"Save",
		"Cancel");
	
	if(a == NX_ALERTDEFAULT) {
	
		// The user wishes to close without saving
		
		return 1;
	
	}	
		
	if(a == NX_ALERTALTERNATE) {
	
		// The user wishes to save before closing
		
		if([self saveDocument:nil] == nil)
			return 0;
		
		return 1;
	}	
		
	if(a == NX_ALERTDEFAULT) {
	
		// The user wants to cancel the close operation
		
		return 0;
	
	}	
	
	return 0;	// should never be reached	
}


- windowWillClose:sender;
{
    // This message is sent from theWindow when user has clicked in
    // the close button of the window. 
    
    // If we return nil, the window will not be closed.
    
    // Check if the document is dirty and if so: prompt the user
    
    	if([theWindow isDocEdited]) {
	
		if([self confirmClose] == 0)
			return nil;
		
	}
	
    // If we are an OOE document, tell client we will close

	if(isOOEServer)
		[self closeEmbeddedDocument:sender];
	
    // Get the list of documents from the global application object
    // and remove myself from the list

	[[NXApp theDocuments] removeObject:self];
	
    // Finally, free the object
	
	return [NXApp delayedFree:self];
}


//
//
//

- closeEmbeddedDocument:sender;
{
    // Tell client that document will close
    
	[theOOEClient OOE_documentWillClose:self];	
   
     // Unregister for application death notification
    
    	[[theOOEClient connectionForProxy] 
		unregisterForInvalidationNotification:self];

    // Very important: remove the object from the
    // Distributed Objects system
	
	[NXConnection removeObject:self];
   
	return self;
}

@end


