//
//	Copyright (c) 1995 Bjorn Backlund.  
//	All rights reserved. 
//

#import "app.h"


@implementation Cosmo_Doc (OOE)


- connectToClient:client;
{
	id connectionToClient;
	const char *path = NULL;
	
    // Remember that the document is an OOE server document
    
	isOOEServer = YES;
	
    // Store pointer to client object
    
	theOOEClient = client;

    // Tell the Distributed Object system that we want to be
    // notified if the client application terminates
    // accidently
	 
    	connectionToClient = [client connectionForProxy];
    	[connectionToClient registerForInvalidationNotification:self];
      	  
    // Optional: to speed things up: tell DO system that
    // we will only send messages in the OOE_ClientInterface
    // protocol
	  
    	[client setProtocolForProxy:@protocol(OOE_ClientInterface)];
	
    // Get the path of the client document so that we can use
    // it in the window title
	
	path = [client OOE_documentPath:self];
	if(path)theOOEClientPath = NXCopyStringBuffer(path);
	
    //
    // Enable the OOE button
    //
    	
	[[theScrollView theOOEButton] setEnabled:YES];
	
	return self;
}

- senderIsInvalid:sender;
{
    // This message is sent to the document when the client
    // application has died accidently
    
    // Reset the pointer to the client object
    
	theOOEClient = nil;
	
    // Add UI stuff here: for example change window title to
    // reflect that the client has died

    //
    // Disable the OOE button
    //
    	
	[[theScrollView theOOEButton] setEnabled:NO];
	
    	return self;
}

//
//
//


- retrieveData:sender;
{
	id pboard = nil;
	NXStream *stream = NULL;
	NXTypedStream *typedStream = NULL;
	const char *ftype = NULL;
	  
    // Get a pointer to the OOE pasteboard
	
	pboard = [Pasteboard newName: OOE_PASTEBOARD];
	if(pboard == nil) {
	
		NXLogError("ERROR: No '%s' pasteboard!",
				TheCosmoOOEPboardType);
		return nil;
	}
	
    // Check that there is data of the Comso type on the
    // pasteboard
	
    	ftype = [pboard findAvailableTypeFrom: &TheCosmoOOEPboardType
				num:1];

	if(ftype == NULL) {
	
		NXLogError("ERROR: No '%s' type in pasteboard '%s'!",
				TheCosmoOOEPboardType, OOE_PASTEBOARD);
		return nil;
	}
	
    // Get a typed stream to the data on the pasteboard
    
 	stream = [pboard readTypeToStream: TheCosmoOOEPboardType];
	typedStream = NXOpenTypedStream(stream, NX_READONLY);
	
    // Read data from the stream
    
    	[self readDataFrom: typedStream];
    
    // Cleanup
    
    	if(typedStream)NXCloseTypedStream(typedStream);
    	if(stream)NXCloseMemory(stream, NX_FREEBUFFER);
	
	return self;
}

//
// OOE Server Document Protocol
//

- (int)OOE_isDocEdited;
{
	return [theWindow isDocEdited]; 
}

//
//
//

- (int)OOE_orderDocumentFront:sender;
{
    // Make the Cosmo app the active one
    
	[NXApp activateSelf:YES];
	
    // Bring the window to the front
	
	[theWindow makeKeyAndOrderFront:nil];
	
	return 1;
}

//
//
//

- (int)OOE_getSize:(NXSize *)aSize;
{
	NXRect aRect;
	
    // Return the bounding box of all polylines
    
    // Calc the bouding box
    
	[self calculateBoundingBox:&aRect];
	
    // Add a margin of 5 pixels
    
    	NXInsetRect(&aRect, -5, -5);
	
    // Return the size
    
	*aSize = aRect.size;
	
	return 1;
}

- calculateBoundingBox: (NXRect *)aRect;
{
	int i, size = [thePolylines count];
	NXRect tmpRect;
	id aPolyline;
	
    // Get bounding box of first polyline
    
    	aPolyline = [thePolylines objectAt: 0];
	[aPolyline calculateBoundingBox: aRect];
	
    // Add bounding boxes for the rest of the polylines
    
    	for(i=1; i<size; i++) {

		aPolyline = [thePolylines objectAt: i];
		
		[aPolyline calculateBoundingBox: &tmpRect];
		
		NXUnionRect(&tmpRect, aRect);

	}
	
	return self;
}

//
//
//

- (void)OOE_closeDocument:sender;
{
    // Reset the OOE server flag
    
	isOOEServer = NO;
	
    // Make sure we are considered non-edited
    
	[theWindow setDocEdited:NO];
	
    // Close the window
    
	[theWindow performClose:nil];
	
	return; 
}

- (int)OOE_document: client savedAs:(const char *)path;
{
    // Free any previous path
    
	if(theOOEClientPath != NULL) 
		NX_FREE(theOOEClientPath);
	
    // Store the path
    	
	if(path == NULL) path = "UNTITLED";
	theOOEClientPath = NXCopyStringBuffer(path);
	
    // In the future: update window title
    
	return 1;
}

//
//
//

- (int)OOE_provideData:sender;
{
	id pboard = nil;
	
    // Get a pointer to the OOE pasteboard
    
	pboard = [Pasteboard newName: OOE_PASTEBOARD];
	
    // Declare the pasteboard with OOE document data and
    // EPS data
    
    	[self declareOOEPasteboard:pboard];
	
    // Write document data (= the polylines) to the pasteboard
	
	[self writeDataToPasteboard: pboard];
	
    // Write EPS to the pasteboard
    
	[self writePostScriptToPasteboard:pboard];

    // Reset the edited flag
    
	[theWindow setDocEdited:NO];
	
	return 1;
}

- declareOOEPasteboard:pboard;
{
	const char *types[2];
	int k = 0;
	
    // Declare pboard
    
    // EPS
    
	types[k++] = NXPostScriptPboardType;
	
    // Cosmo internal document data
    
	types[k++] = TheCosmoOOEPboardType;
	
    // Declare
    
	[pboard declareTypes: types 
		num: k 
		owner: NXApp];	
	
	return self;
}

- writeDataToPasteboard:pboard;
{
	NXStream *stream;
	NXTypedStream *typedStream;
	
    // Open a stream to write to
    
   	stream = NXOpenMemory(NULL, 0, NX_WRITEONLY);
	typedStream = NXOpenTypedStream(stream, NX_WRITEONLY);
	
    // Write data to stream
    
	[self writeDataTo: typedStream];
	
    // Copy stream data to pasteboard
    
    	[pboard writeType: TheCosmoOOEPboardType
		fromStream: stream];
	
    // Cleanup
    
    	if(typedStream)NXCloseTypedStream(typedStream);
    	if(stream)NXCloseMemory(stream, NX_FREEBUFFER);
    	
	return self;
}

- writePostScriptToPasteboard: pboard;
{
	NXRect aRect;
	NXStream *stream = NULL;
	
    // Get bouding box of all polylines
    
    	[self calculateBoundingBox: &aRect];

    // Add a margin of 5 pixels
    
    	NXInsetRect(&aRect, -5, -5);
	
    // Open a stream to copy EPS into
    
    	stream = NXOpenMemory(NULL, 0, NX_READWRITE);
	
    // Let view produce EPS
    
    	[theView copyPSCodeInside: &aRect to:stream];

     // Copy stream data to pasteboard
    
    	[pboard writeType: NXPostScriptPboardType
		fromStream: stream];
	
    // Cleanup
    
    	if(stream)NXCloseMemory(stream, NX_FREEBUFFER);
	
	return self;
}

//
// File Filtering
//

- declareFileFilterOOEPasteboard:aPasteboard;
{
	const char *types[3];
	int k = 0;
	
    // Declare pboard
    
    // EPS
    
	types[k++] = NXPostScriptPboardType;
	
    // Cosmo internal document data
    
	types[k++] = TheCosmoOOEPboardType;
	
    // OOE source application information
    
	types[k++] = TheOOE_ApplicationInfoPboardType;
	
    // Declare
    
	[aPasteboard declareTypes: types 
		num: k 
		owner: NXApp];	
	
	return self;
}

- addToFileFilterOOEPasteboard:aPasteboard;
{
    // Declare pasteboard with OOE data, OOE source application
    // info and Encapsulated PostScript
	
	[self declareFileFilterOOEPasteboard:aPasteboard];
	
    // Add OOE
    
    	[self writeDataToPasteboard:aPasteboard];
	
    // Add EPS
 
    	[self writePostScriptToPasteboard:aPasteboard];
	
    // Add OOE source application information
    
    	[theView writeOOESourceApplicationToPasteboard:aPasteboard];
    	
	return self;
}

//
//
//

- orderClientDocumentFront:sender;
{
	[theOOEClient OOE_orderClientDocumentFront:self];

	return self;
}

@end


