//
//	Copyright (c) 1995 Bjorn Backlund.  
//	All rights reserved. 
//

#import "app.h"


@implementation Cosmo_Doc (Saving)

- (const char *)promptForFileName;
{
	id savePanel;
	int ok;
	
    // Prompt the user for a file name,
    // use the NEXTSTEP builtin SavePanel
	
    // Get the SavePanel object
	
	savePanel = [SavePanel new];
	
    // Set the file extension we want our filenames to end with
	
	[savePanel setRequiredFileType: "cosmo"];
	
    // Open panel and let user enter/select a file name
	
	ok = [savePanel runModal];
	
    // Check if user cancelled the operation
	
	if(ok != NX_OKTAG) return NULL;
	
    // Return the selected file name
	
	return [savePanel filename];
}


- saveDocumentTo: (const char *)file;
{
	NXTypedStream *s;
	
    // Open a stream on the file
	
	s = NXOpenTypedStreamForFile(file, NX_WRITEONLY);
	
	if(s == NULL) {
		
		NXLogError("ERROR: Could not open stream for %s.", file);
		return nil;
	}
	
    // Write the polylines to the stream
    
    	[self writeDataTo:s];

    // Close the stream = data gets written to file
    
    	NXCloseTypedStream(s);
	
    // NOTE: the NXCloseTypedStream might raise an exception
    // but we don't care for simplitcily
	
	return self;
}

- writeDataTo: (NXTypedStream *)s;
{
    // Write the polylines to the stream
    
    	NXWriteRootObject(s, thePolylines);

	return self;
}



- saveDocument:sender;
{
	const char *file = thePath;
	id r;
	
    // If the document is an OOE server document
    // call saveEmbeddedDocument:
	
	if(isOOEServer) 
		return [self saveEmbeddedDocument: sender];
	
    // Check if we have saved earlier
    
    	if(thePath == NULL) {
	
		file = [self promptForFileName];
		if(file == NULL)
			return nil;
	}
	
    // Save the document to selected file;
	
	r = [self saveDocumentTo: file];
	
	if(r == nil) return nil;
	
    // Copy and store selected file name 
    
    	if(thePath == NULL)
		thePath = NXCopyStringBuffer(file);
    
    // Reset the dirty bit
    
    	[theWindow setDocEdited:0];
    
	return self;
}

- saveEmbeddedDocument:sender;
{
    // Send message to client object
    
	[theOOEClient OOE_documentWasChanged:self];
	
	return self;
}


@end


