//
//	Copyright (c) 1995 Bjorn Backlund.  
//	All rights reserved. 
//

#import "app.h"


@implementation Cosmo_View (MouseHandling)

- mouseDown: (NXEvent *)anEvent;
{
    // This method is "automagically" sent to the 
    // view when the mouse is clicked within
    // the view
    
    // Check which state we are in
    
    // If in the normal state, find a polyline to select
	
	if(theState == 0) {
	
		[self mouseDownInSelectMode:anEvent];
		return self;
	}
	
    // If in the "create state", call the mouseDownInCreateMode method
	
	if(theState == 1) {
	
		[self mouseDownInCreateMode:anEvent];
		return self;
	}
	
	return self;
}

- mouseDownInSelectMode: (NXEvent *)anEvent;
{
	NXPoint mousePoint;
	id aPolyline;
	
    // Get the location of the mouse in view coordinates
	
	mousePoint = anEvent->location;
	[self convertPoint: &mousePoint fromView:nil];
	
    // Find the polyline with a point near the mouse point
    
    	aPolyline = [self findPolylineAtPoint: &mousePoint];
	
	if(aPolyline != theSelectedPolyline) {
	
	    // remember the selected polyline
	    
		theSelectedPolyline = aPolyline;
			
	    // redisplay the view to show the new selection
	    
		[self display];
		
	}
	
	return self;
}

- mouseDownInCreateMode: (NXEvent *)anEvent;
{
	int numberOfClicks;
	NXPoint mousePoint;
	id newPoint;
	id polylines;
	id newest;
	
    // Get the number of clicks
	
	numberOfClicks = anEvent->data.mouse.click;
	
    // If the number of  mouse clicks is two, 
    // the user wish to exit the "create state"
	
	if(numberOfClicks == 2) {
	
		theState = 0;
		return self;
	
	}
	
    // When we come here, we have a single click, 
    // get the location of the mouse in view coordinates
	
	mousePoint = anEvent->location;
	[self convertPoint: &mousePoint fromView:nil];
	
    // Create a new point object and set the coordinates of the point object
    // to the coordinates of the mouse
	
	newPoint = [[Cosmo_Point allocFromZone:[self zone]] init];
	[newPoint setTheX: mousePoint.x];
	[newPoint setTheY: mousePoint.y];
	
    // Get the newest polyline (the last in the list of polylines)
	
	polylines = [theDocument thePolylines];
	newest = [polylines lastObject];
	
    // Add the point object to the polyline object

	[newest addObject: newPoint];
	
    // Redisplay myself
	
	[self display];
	
	return self;
}

@end


