
#import "draw.h"

// OOE, Moved these from .m file
//
#define DRAW_VERSION_2_0 184
#define DRAW_VERSION_3_0_PRERELEASE 234
#define DRAW_VERSION_3_0 245
#define NEW_DRAW_VERSION DRAW_VERSION_3_0



// OOE
@class GraphicView, UndoText;
//

// OOE
extern Window *createWindowFor(View *view, NXRect *windowContentRect, const char *frameString);


@interface DrawDocument : ChangeManager
{
    GraphicView *view;		/* the document's GraphicView */
    Window *window;		/* the window the GraphicView is in */
    PrintInfo *printInfo;	/* the print information for the GraphicView */
    char *name;			/* the name of the document */
    char *directory;		/* the directory it is in */
    char *iconPathList;		/* list of files last dragged over document */
    BOOL haveSavedDocument;	/* whether document has associated disk file */
    UndoText *undoFieldEditor;
    NXDataLinkManager *linkManager;	/* manager of data links */
    
    #ifdef OOE_SERVER
    BOOL	isOOEServer;
    id		theOOEClientProxy;
    char	*theOOEClientPath;
    #endif
}

// OOE

#ifndef DRAW_FILTER		// OOE 2.0
+ createDoc;
+ new;
#endif

+ (NXZone *)newZone;	
	
//


/* Very private instance method needed by factory methods */

- (BOOL)loadDocument:(NXStream *)stream 
	frameSize:(NXRect *)frame 
	frameString:(char *)frameString;

/* Factory methods */

+ newFromStream:(NXStream *)stream;
+ newFromFile:(const char *)file andDisplay:(BOOL)display;
+ newFromFile:(const char *)file;

/* Public methods */

- init;
- free;
- close;	/* Frees (delayed). */

/* Data link methods */

#ifndef DRAW_FILTER		// OOE 2.0
- setLinkManager:(NXDataLinkManager *)aLinkManager;
- showSelection:(NXSelection *)selection;
- copyToPasteboard:(Pasteboard *)pasteboard at:(NXSelection *)selection cheapCopyAllowed:(BOOL)flag;
- pasteFromPasteboard:(Pasteboard *)pasteboard at:(NXSelection *)selection;
- importFile:(const char *)filename at:(NXSelection *)selection;
- windowForSelection:(NXSelection *)selection;
#endif



/* Overridden from ChangeManager */

#ifndef DRAW_FILTER		// OOE 2.0
- changeWasDone;
- changeWasUndone;
- changeWasRedone;
- clean:sender;
- dirty:sender;
#endif


/* Public Methods */

#ifndef DRAW_FILTER		// OOE 2.0
- resetScrollers;
#endif

- view;
- printInfo;

/* Target/Action methods */

#ifndef DRAW_FILTER		// OOE 2.0
- changeLayout:sender;
- changeGrid:sender;
- save:sender;
- saveAs:sender;
- saveTo:sender;
- revertToSaved:sender;
- showTextRuler:sender;
- hideRuler:sender;
- close:sender;	// OOE, added this one
#endif



/* Private method used by saveTo: method */

#ifndef DRAW_FILTER		// OOE 2.0
- changeSaveType:sender;
#endif

- (BOOL)isSameAs:(const char *)filename;


/* Document name and file handling methods */

- (const char *)filename;
- (const char *)directory;
- (const char *)name;
- setName:(const char *)name andDirectory:(const char *)directory;
- (BOOL)setName:(const char *)name;
- setTemporaryTitle:(const char *)title;


#ifndef DRAW_FILTER		// OOE 2.0
- saveTo:(const char *)type using:(SEL)streamWriter;
- save;
#endif



/* Services menu methods */

#ifndef DRAW_FILTER		// OOE 2.0
- registerForServicesMenu;
- validRequestorForSendType:(NXAtom)sendType andReturnType:(NXAtom)returnType;
- writeSelectionToPasteboard:pboard types:(NXAtom *)types;
#endif


/* Window delegate methods */

#ifndef DRAW_FILTER		// OOE 2.0
- windowWillClose:sender cancellable:(BOOL)flag;
- windowWillClose:(Window *)sender;
- windowDidBecomeMain:(Window *)sender;
- windowWillResize:(Window *)sender toSize:(NXSize *)size;
- windowWillMiniaturize:(Window *)sender toMiniwindow:counterpart;
- windowWillReturnFieldEditor:(Window *)sender toObject:client;
#endif


/* Menu command validation method */

#ifndef DRAW_FILTER		// OOE 2.0
- (BOOL)validateCommand:(MenuCell *)menuCell;
#endif



/* Cursor setting */

#ifndef DRAW_FILTER		// OOE 2.0
- resetCursor;
#endif




@end

