



typedef enum { Normal, Resizing } DrawStatusType;
typedef enum { LEFT = 1, RIGHT, BOTTOM, TOP, HORIZONTAL_CENTERS, VERTICAL_CENTERS, BASELINES } AlignmentType;

extern DrawStatusType DrawStatus;
extern NXAtom DrawPboardType;
extern BOOL InMsgPrint;

typedef enum { ByRect, ByGraphic, ByList } DrawSelectionType;


#define LEFTARROW	172
#define RIGHTARROW	174
#define UPARROW		173
#define DOWNARROW	175

extern float KeyMotionDeltaDefault;





/* Code-cleaning macros */		// OOE 2.0, moved here from .m file

#define stopTimer(timer) if (timer) { \
    NXEndTimer(timer); \
    timer = NULL; \
}

#define startTimer(timer) if (!timer) timer = NXBeginTimer(NULL, 0.1, 0.1);

#define GRID (gvFlags.gridDisabled ? 1.0 : (gvFlags.grid ? (NXCoord)gvFlags.grid : 1.0))

#define grid(point) \
    (point).x = floor(((point).x / GRID) + 0.5) * GRID; \
    (point).y = floor(((point).y / GRID) + 0.5) * GRID;


extern void getRegion(NXRect *region, const NXPoint *p1, const NXPoint *p2);
extern BOOL checkForGroup(List *list);





/* Update modes for links. */

#define UPDATE_NEVER 0
#define UPDATE_IMMEDIATELY -1
#define UPDATE_NORMALLY 1





@interface GraphicView : View
{
    List *glist;			/* the list of Graphics */
    List *slist;			/* the list of selected Graphics. In 
    					   Draw with undo we are very careful
					   to keep slist sorted, like glist */
    Window *cacheWindow;		/* the off-screen cache */
    struct {
#ifdef __BIG_ENDIAN__
	unsigned int groupInSlist:1;	/* whether a Group is in the slist */
	unsigned int cacheing:1;	/* whether cacheing or drawing */
	unsigned int suspendLinkUpdate:1; /* don't update links */
	unsigned int grid:8;		/* grid size */
	unsigned int showGrid:1;	/* whether grid is visible */
	unsigned int locked:1;		/* some graphics are locked */
	unsigned int gridDisabled:1;	/* whether grid is enabled */
	unsigned int freeOriginalPaste:1;/* whether originalPaste needs free */
	unsigned int serviceActsOnSelection:1;	/* whether service acts on selection */
#else
	unsigned int serviceActsOnSelection:1;	/* whether service acts on selection */
	unsigned int freeOriginalPaste:1;/* whether originalPaste needs free */
	unsigned int gridDisabled:1;	/* whether grid is enabled */
	unsigned int locked:1;		/* some graphics are locked */
	unsigned int showGrid:1;	/* whether grid is visible */
	unsigned int grid:8;		/* grid size */
	unsigned int suspendLinkUpdate:1; /* don't update links */
	unsigned int cacheing:1;	/* whether cacheing or drawing */
	unsigned int groupInSlist:1;	/* whether a Group is in the slist */
#endif
// These last 16 bits are NOT archived, so don't depend on them being permanent across a Save/Open
	unsigned int selectAll:1;	/* select all was last select operation performed */
	unsigned int dragCopyOk:1;	/* true if dragging with sourcMask == copy is ok */
	unsigned int dragLinkOk:1;	/* true if dragging with sourcMask == link is ok */
	unsigned int padding:13;
    } gvFlags;
    short *gupCoords;			/* points in the grid user path */
    int gupLength;			/* number of points in gupCoords */
    char *gupOps;			/* movetos and linetos in the gup */
    short *gupBBox;			/* bounding box of the gup */
    float gridGray;			/* grayness of the grid */
    int consecutivePastes;		/* number of consecutive pastes */
    int lastPastedChangeCount;		/* the change count of last paste */
    int lastCopiedChangeCount;		/* the change count of last cut or copy */
    int lastCutChangeCount;		/* the change count of last cut */
    View *editView;			/* flipped subview for editing */
    Graphic *originalPaste;		/* the first pasted graphic */
    NXDataLinkManager *linkManager;	/* manager of data links */
    NXRect *invalidRect;		/* invalid area which must be cleaned up */
    NXRect *dragRect;			/* last rectangle we dragged out to select */
    Storage *linkTrackingRects;		/* the rects of the links we are tracking */
}

/* Class initialization */

+ initialize;




/* Creation methods */

- initFrame:(const NXRect *)frameRect;



/* Free method */

- free;



/* Public methods */

- (BOOL)isEmpty;
- (BOOL)hasEmptySelection;
- dirty;
- getSelection;
- setGroupInSlist:(BOOL)setting;
- resetGroupInSlist;
- resetLockedFlag;

- redrawGraphics:graphicsList afterChangeAgent:changeAgent performs:(SEL)aSelector;
- graphicsPerform:(SEL)aSelector;
- graphicsPerform:(SEL)aSelector with:(void *)argument;

- cache:(const NXRect *)rect;
- cache:(const NXRect *)rect andUpdateLinks:(BOOL)updateLinks;
- selectionCache;


- (Graphic *)selectedGraphic;
- (List *)selectedGraphics;
- (List *)graphics;


#ifndef DRAW_FILTER		// OOE 2.0
- placeGraphic:(Graphic *)graphic at:(const NXPoint *)location;
- removeGraphic:(Graphic *)graphic;
- insertGraphic:(Graphic *)graphic;
#endif


/* Methods overridden from superclass */

- sizeTo:(NXCoord)width :(NXCoord)height;



/* Getting/Setting the current Graphic */

- (Graphic *)currentGraphic;
- setCurrentGraphic:sender;



/* Writing Draw Forms */

- (BOOL)hasFormEntries;
- writeFormEntriesToFile:(const char *)filename;




/* Accepting becoming the First Responder */

- (BOOL)acceptsFirstResponder;



/* Printing-related methods */

- beginSetup;



/* Archiving methods */

- awake;
- write:(NXTypedStream *)stream;
- read:(NXTypedStream *)stream;

@end







/*
 * Since we can't be sure that we have an InspectorPanel, we use the
 * objective-C respondsTo: mechanism to see if we can send the message
 * initializeGraphic: to [NXApp inspectorPanel].  This dummy interface
 * declaration declares those messages (so that even if they don't exists,
 * we can at least use them to check with respondsTo:).  We don't want
 * to import DrawApp.h or InspectorPanel.h since we might accidentally
 * introduce a dependency on them which wouldn't be caught because we
 * imported both of their interfaces.
 */


@interface PossibleInspectorPanel : Object

- inspectorPanel;
- initializeGraphic:(Graphic *)graphic;

@end
