
#ifndef DRAW_FILTER		// OOE 2.0


#import "draw.h"

#import "GraphicView_C_Align.h"

@implementation GraphicView (Align)

/* Alignment methods */

+ (SEL)actionFromAlignType:(AlignmentType)alignType
{
    switch (alignType) {
	case LEFT: return @selector(moveLeftEdgeTo:);
	case RIGHT: return @selector(moveRightEdgeTo:);
	case BOTTOM: return @selector(moveBottomEdgeTo:);
	case TOP: return @selector(moveTopEdgeTo:);
	case HORIZONTAL_CENTERS: return @selector(moveVerticalCenterTo:);
	case VERTICAL_CENTERS: return @selector(moveHorizontalCenterTo:);
	case BASELINES: return @selector(moveBaselineTo:);
    }
    return (SEL)NULL;
}

- align:sender
{
    [self alignBy:(AlignmentType)[[sender selectedCell] tag]];
    return self;
}


- alignGraphicsBy:(AlignmentType)alignType edge:(NXCoord *)edge
{
    SEL	action;
    id change;
    
    change = [[AlignGraphicsChange alloc] initGraphicView:self];
    [change startChange];
        action = [GraphicView actionFromAlignType:alignType];
	[self graphicsPerform:action with:edge];
    [change endChange];

    return self;
}

- alignBy:(AlignmentType)alignType
{
    int i;
    NXRect rect;
    Graphic *graphic;
    NXCoord minEdge = 10000.0;
    NXCoord maxEdge = 0.0;
    NXCoord baseline = 0.0;

    for (i = [slist count]-1; i >= 0 && !baseline; i--) {
	graphic = [slist objectAt:i];
	[graphic getBounds:&rect];
	switch (alignType) {
	    case LEFT:
	 	if (rect.origin.x < minEdge) 
		    minEdge = rect.origin.x;
	        break;
	    case RIGHT:
		if (rect.origin.x + rect.size.width > maxEdge) 
		    maxEdge = rect.origin.x + rect.size.width;
	        break;
	    case BOTTOM:
		if (rect.origin.y < minEdge) 
		    minEdge = rect.origin.y;
	        break;
	    case TOP:
		if (rect.origin.y + rect.size.height > maxEdge) 
		    maxEdge = rect.origin.y + rect.size.height;
	        break;
	    case HORIZONTAL_CENTERS:
		if (rect.origin.y + floor(rect.size.height / 2.0) < minEdge)
		    minEdge = rect.origin.y + floor(rect.size.height / 2.0);
	        break;
	    case VERTICAL_CENTERS:
		if (rect.origin.x + floor(rect.size.width / 2.0) < minEdge)
		    minEdge = rect.origin.x + floor(rect.size.width / 2.0);
	        break;
	    case BASELINES:
		baseline = [graphic baseline];
	        break;
	}
    }

    switch (alignType) {
        case LEFT:
        case BOTTOM:
        case HORIZONTAL_CENTERS:
        case VERTICAL_CENTERS:
	    [self alignGraphicsBy:alignType edge:&minEdge];
	    break;
        case RIGHT:
        case TOP:
	    [self alignGraphicsBy:alignType edge:&maxEdge];
	    break;
	case BASELINES:
	    if (baseline) [self alignGraphicsBy:alignType edge:&baseline];
    }
    [window flushWindow];

    return self;
}

@end


#endif

