
#import "draw.h"

#import "GraphicView_C_Drawing.h"

@implementation GraphicView (Drawing)

- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the GraphicView.
 *
 * If cacheing is on or if NXDrawingStatus != NX_DRAWING, then all the
 * graphics which intersect the specified rectangles will be drawn (and
 * clipped to those rectangles).  Otherwise, the specified rectangles
 * are composited to the screen from the off-screen cache.  The invalidRect
 * stuff is to clean up any temporary drawing we have in the view
 * (currently used only to show a source selection in the links mechanism--
 * see showSelection: in gvLinks.m).
 */
{
    NXRect *rp;
    NXRect r, visibleRect;
    int i, j, gstate;

    if (rects == NULL) return self;

    if (!gvFlags.cacheing && invalidRect && (rects != invalidRect)) {
	NXUnionRect(rects, invalidRect);
	[self drawSelf:invalidRect :1];
	[window flushWindow];
	NX_FREE(invalidRect);
	invalidRect = NULL;
	return self;
    }

    if (gvFlags.cacheing || NXDrawingStatus != NX_DRAWING) {
	if (NXDrawingStatus == NX_DRAWING) {
	    [[cacheWindow contentView] lockFocus];
	    NXRectClip(rects);
	    PSsetgray(NX_WHITE);
	    for (j = (rectCount > 1) ? 1 : 0; j < rectCount; j++) {
		NXRectFill(&rects[j]);
		if (gvFlags.showGrid && gvFlags.grid >= 4) {
		    PSsetlinewidth(0.0);
		    PSsetgray(gridGray);
		    DPSDoUserPath(gupCoords, gupLength, dps_short,
				  gupOps, gupLength >> 1, gupBBox, dps_ustroke);
		    PSsetgray(NX_WHITE);
		}
	    }
	}
	for (j = (rectCount > 1) ? 1 : 0; j < rectCount; j++) {
	    i = [glist count];
	    while (i--) [[glist objectAt:i] draw:&rects[j]];
	}
	[Graphic showFastKnobFills];
	if (NXDrawingStatus == NX_DRAWING) {
	    [[cacheWindow contentView] unlockFocus];
	}
    }

    if (!gvFlags.cacheing && NXDrawingStatus == NX_DRAWING) {
	gstate = [cacheWindow gState];
	[self getVisibleRect:&visibleRect];
	for (j = 0; j < rectCount; j++) {
	    rp = &r;
	    r = rects[j];
	    if (!NXEqualRect(rp, &visibleRect)) rp = NXIntersectionRect(&visibleRect, rp);
	    if (rp) {
		PScomposite(NX_X(rp), NX_Y(rp), NX_WIDTH(rp), NX_HEIGHT(rp), gstate,
			    NX_X(rp), NX_Y(rp), NX_COPY);
	    }
	}
    }

    return self;
}

@end


