#ifndef DRAW_FILTER		// OOE 2.0

#import "draw.h"

#import "GraphicView_C_Grid.h"

@implementation GraphicView (Grid)

- alignToGrid:sender 
{
    id change;
    
    change = [[AlignGraphicsChange alloc] initGraphicView:self];
    [change startChange];
	[self graphicsPerform:@selector(alignToGrid:) with:self];
	[window flushWindow];
    [change endChange];
    
    return self;
}

- sizeToGrid:sender
{
    id change;

    change = [[DimensionsGraphicsChange alloc] initGraphicView:self];
    [change startChange];
	[self graphicsPerform:@selector(sizeToGrid:) with:self];
	[window flushWindow];
    [change endChange];
    
    return self;
}

- enableGrid:sender
/*
 * If the tag of the sender is non-zero, then gridding is enabled.
 * If the tag is zero, then gridding is disabled.
 */
{
    [self setGridEnabled:[sender selectedTag] ? YES : NO];
    return self;
}

- hideGrid:sender
/*
 * If the tag of the sender is non-zero, then the grid is made visible
 * otherwise, it is hidden (but still conceivable in effect).
 */
{
    [self setGridVisible:[sender selectedTag] ? YES : NO];
    return self;
}

- takeGridValueFrom:sender
{
    [self setGridSpacing:[sender intValue]];
    return self;
}

- takeGridGrayFrom:sender
{
    [self setGridGray:[sender floatValue]];
    return self;
}

//
//
//

/* Methods to modify the grid of the GraphicView. */

- (int)gridSpacing
{
    return gvFlags.grid;
}

- (BOOL)gridIsVisible
{
    return gvFlags.showGrid;
}

- (BOOL)gridIsEnabled
{
    return !gvFlags.gridDisabled;
}

- (float)gridGray
{
    return gridGray;
}

- setGridSpacing:(int)gridSpacing
{
    id change;
    
    if (gridSpacing != gvFlags.grid && gridSpacing > 0 && gridSpacing < 256) {
	change = [[GridChange alloc] initGraphicView:self];
	[change startChange];
	    gvFlags.grid = gridSpacing;
	    if (gvFlags.showGrid) {
		[self resetGUP];
		[self cache:&bounds andUpdateLinks:NO];
		[window flushWindow];
	    }
	[change endChange];
    }

    return self;
}

- setGridEnabled:(BOOL)flag
{
    id change;
    
    change = [[GridChange alloc] initGraphicView:self];
    [change startChange];
        gvFlags.gridDisabled = flag ? NO : YES;
    [change endChange];

    return self;
}

- setGridVisible:(BOOL)flag
{
    id change;
    
    if (gvFlags.showGrid != flag) {
	change = [[GridChange alloc] initGraphicView:self];
	[change startChange];
	    gvFlags.showGrid = flag;
	    if (flag) [self resetGUP];
	    [self cache:&bounds andUpdateLinks:NO];
	    [window flushWindow];
	[change endChange];
    }

    return self;
}

- setGridGray:(float)gray
{
    id change;
    
    if (gray != gridGray) {
	change = [[GridChange alloc] initGraphicView:self];
	[change startChange];
	    gridGray = gray;
	    if (gvFlags.showGrid) {
		[self cache:&bounds andUpdateLinks:NO];
		[window flushWindow];
	    }
	[change endChange];
    }

    return self;
}

- setGridSpacing:(int)gridSpacing andGray:(float)gray
{
    id change;
    
    if (gray != gridGray || (gridSpacing != gvFlags.grid && gridSpacing > 0 && gridSpacing < 256)) {
	change = [[GridChange alloc] initGraphicView:self];
	[change startChange];
	    gridGray = gray;
	    if (gvFlags.grid != gridSpacing && gridSpacing > 0 && gridSpacing < 256) {
		gvFlags.grid = gridSpacing;
		if (gvFlags.showGrid) [self resetGUP];
	    }
	    if (gvFlags.showGrid) {
		[self cache:&bounds andUpdateLinks:NO];
		[window flushWindow];
	    }
	[change endChange];
    }

    return self;
}

- grid:(NXPoint *)p
{
    grid(*p);
    return self;
}


@end




#endif

