#ifndef DRAW_FILTER		// OOE 2.0

#import "draw.h"

#import "GraphicView_C_KeyEvents.h"

@implementation GraphicView (KeyEvents)


- keyDown:(NXEvent *)event
/*
 * Handles one of the arrow keys being pressed.
 * Note that since it might take a while to actually move the selection
 * (if it is large), we check to see if a bunch of arrow key events have
 * stacked up and move them all at once.
 */
{
    NXPoint p;
    NXEvent e;
    NXCoord delta;
    BOOL gotOne, first;
    NXEvent* eptr = event;

    if ((event->data.key.charSet != NX_ASCIISET ||
	 event->data.key.charCode != 127) &&
	(event->data.key.charSet != NX_SYMBOLSET ||
	 (event->data.key.charCode != LEFTARROW &&
	  event->data.key.charCode != RIGHTARROW &&
	  event->data.key.charCode != DOWNARROW &&
	  event->data.key.charCode != UPARROW))) {
	return [super keyDown:event];
    }

    if (event->data.key.charSet == NX_ASCIISET) return [self delete:self];

    p.x = p.y = 0.0;
    delta = KeyMotionDeltaDefault;
    delta = floor(delta / GRID) * GRID;
    delta = MAX(delta, GRID);

    first = YES;
    do {
	gotOne = NO;
	if (eptr->data.key.charSet == NX_SYMBOLSET) {
	    switch (eptr->data.key.charCode) {
		case LEFTARROW:
		    p.x -= delta;
		    gotOne = YES;
		    break;
		case RIGHTARROW:
		    p.x += delta;
		    gotOne = YES;
		    break;
		case UPARROW:
		    p.y += delta;
		    gotOne = YES;
		    break;
		case DOWNARROW:
		    p.y -= delta;
		    gotOne = YES;
		    break;
		default:
		    break;
	    }
	}
	if (eptr && gotOne && !first) [NXApp getNextEvent:NX_KEYDOWNMASK];
	first = NO;
    } while (gotOne && (eptr = [NXApp peekNextEvent:NX_KEYDOWNMASK into:&e]));

    if (p.x || p.y) {
	[self moveGraphicsBy:&p andDraw:YES];
	[[self window] flushWindow];
	NXPing();
    }

    return self;
}

@end

#endif


