#ifndef DRAW_FILTER		// OOE 2.0

#import "draw.h"

#import "GraphicView_C_Layout.h"

@implementation GraphicView (Layout)

- lock:sender
/*
 * Locks all the items in the selection so that they can't be selected
 * or resized or moved.  Useful if there are some Graphics which are getting
 * in your way.  Undo this with unlock:.
 */
{
    id change;

    if ([slist count] > 0) {
	change = [[LockGraphicsChange alloc] initGraphicView:self];
	[change startChange];
	    gvFlags.locked = YES;
	    [slist makeObjectsPerform:@selector(lock)];
	    [self deselectAll:sender];
	[change endChange];
    }

    return self;
}

- unlock:sender
{
    id change;

    change = [[UnlockGraphicsChange alloc] initGraphicView:self];
    [change startChange];
	[glist makeObjectsPerform:@selector(unlock)];
	gvFlags.locked = NO;
    [change endChange];

    return self;
}

- bringToFront:sender
/*
 * Brings each of the items in the slist to the front of the glist.
 * The item in the front of the slist will be the new front element
 * in the glist.
 */
{
    id change;
    int i;

    i = [slist count];
    if (i) {
	change = [[BringToFrontGraphicsChange alloc] initGraphicView:self];
	[change startChange];
	    while (i--) [glist insertObject:[glist removeObject:[slist objectAt:i]] at:0];
	    [self recacheSelection];
	[change endChange];
    }

    return self;
}

- sendToBack:sender
{
    int i, count;
    id change;

    count = [slist count];
    if (count > 0) {
	change = [[SendToBackGraphicsChange alloc] initGraphicView:self];
	[change startChange];
	    for (i = 0; i < count; i++) [glist addObject:[glist removeObject:[slist objectAt:i]]];
	    [self recacheSelection];
	[change endChange];
    }

    return self;
}


@end


#endif


