#ifndef DRAW_FILTER		// OOE 2.0

#import "draw.h"

#import "GraphicView_C_Move.h"

@implementation GraphicView (Move)


- (BOOL)move:(NXEvent *)event
/*
 * Moves the selection by cacheing the selected graphics into the
 * selection cache, then compositing them repeatedly as the user
 * moves the mouse.  The tracking loop uses TIMER events to autoscroll
 * at regular intervals.  TIMER events do not have valid mouse coordinates,
 * so the last coordinates are saved and restored when there is a TIMER event.
 */
{
    int gstate;
    NXEvent peek;
    NXCoord dx, dy;
    NXTrackingTimer *timer = NULL;
    NXPoint p, start, last, sboundspad;
    NXRect minbounds, sbounds, startbounds, visibleRect;
    BOOL canScroll, tracking = YES, alternate, horizConstrain = NO, vertConstrain = NO, hideCursor;

    last = event->location;
    alternate = (event->flags & NX_ALTERNATEMASK) ? YES : NO;

    event = [NXApp getNextEvent:MOVE_MASK];
    if (event->type == NX_MOUSEUP) return NO;

    hideCursor = NXGetDefaultValue([NXApp appName], "HideCursorOnMove") ? YES : NO;
    if (hideCursor) PShidecursor();

    [self convertPoint:&last fromView:nil];
    [self grid:&last];

    [self lockFocus];

    gstate = [self cacheSelection];
    gvFlags.suspendLinkUpdate = YES;	/* we'll update links when the move is complete */
    [self graphicsPerform:@selector(deactivate)];
    gvFlags.suspendLinkUpdate = NO;
    [self getBBox:&sbounds of:slist];
    startbounds = sbounds;
    [self getBBox:&minbounds of:slist extended:NO];
    sboundspad.x = minbounds.origin.x - sbounds.origin.x;
    sboundspad.y = minbounds.origin.y - sbounds.origin.y;
    [self compositeSelection:&sbounds from:gstate];

    [self getVisibleRect:&visibleRect];
    canScroll = !NXEqualRect(&visibleRect, &bounds);

    start = sbounds.origin;

    while (tracking) {
	p = event->location;
	[self convertPoint:&p fromView:nil];
	[self grid:&p];
	dx = p.x - last.x;
	dy = p.y - last.y;
	if (dx || dy) {
	    [self drawSelf:&sbounds :1];
	    if (alternate && (dx || dy)) {
		if (ABS(dx) > ABS(dy)) {
		    horizConstrain = YES;
		    dy = 0.0;
		} else {
		    vertConstrain = YES;
		    dx = 0.0;
		}
		alternate = NO;
	    } else if (horizConstrain) {
		dy = 0.0;
	    } else if (vertConstrain) {
		dx = 0.0;
	    }
	    NXOffsetRect(&sbounds, dx, dy);
	    minbounds.origin.x = sbounds.origin.x + sboundspad.x;
	    minbounds.origin.y = sbounds.origin.y + sboundspad.y;
	    [self tryToPerform:@selector(updateRulers:) with:(void *)&minbounds];
	    if (!canScroll || NXContainsRect(&visibleRect, &sbounds)) {
		[self compositeSelection:&sbounds from:gstate];
		stopTimer(timer);
	    }
	    last = p;
	}
	tracking = (event->type != NX_MOUSEUP);
	if (tracking) {
	    if (canScroll && !NXContainsRect(&visibleRect, &sbounds)) {
		[window disableFlushWindow];
		[self scrollPointToVisible:&p]; // actually we want to keep the "edges" of the
						// Graphic being resized that were visible when
						// the move started visible throughout the
						// moving session
		[self getVisibleRect:&visibleRect];
		[self compositeSelection:&sbounds from:gstate];
		[[window reenableFlushWindow] flushWindow];
		startTimer(timer);
	    }
	    p = event->location;
	    if (![NXApp peekNextEvent:MOVE_MASK into:&peek]) {
		event = [NXApp getNextEvent:MOVE_MASK|NX_TIMERMASK];
	    } else {
		event = [NXApp getNextEvent:MOVE_MASK];
	    }
	    if (event->type == NX_TIMER) event->location = p;
	}
    }

    if (canScroll) stopTimer(timer);

    if (hideCursor) PSshowcursor();

    p.x = sbounds.origin.x - start.x;
    p.y = sbounds.origin.y - start.y;
    if (p.x || p.y)
        [self moveGraphicsBy:&p andDraw:NO];

    gvFlags.suspendLinkUpdate = YES;	/* we'll update links when the move is complete */
    [self graphicsPerform:@selector(activate)];
    gvFlags.suspendLinkUpdate = NO;
    NXUnionRect(&sbounds, &startbounds);
    [self updateTrackedLinks:&startbounds];

    [self tryToPerform:@selector(updateRulers:) with:nil];

    [window flushWindow];
    [self unlockFocus];

    return YES;
}

- moveGraphicsBy:(NXPoint *)vector andDraw:(BOOL)drawFlag
{
    id change;

    change = [[MoveGraphicsChange alloc] initGraphicView:self vector:vector];
    [change startChange];
	if (drawFlag) {
	    [self graphicsPerform:@selector(moveBy:) with:(id)vector];
	} else {
	    [slist makeObjectsPerform:@selector(moveBy:) with:(id)vector];
	}
    [change endChange];

    return self;
}

@end

#endif


