//
//	Xanthus
//	Copyright (c) 1992-95 Bjorn Backlund.  
//	All rights reserved. 
//

#import "draw.h"
#import "GraphicView_C_Pasteboard_Functions.h"

/* Methods to search through Pasteboard types lists. */

BOOL IncludesType(const NXAtom *types, NXAtom type)
{
    if (types) while (*types) if (*types++ == type) return YES;
    return NO;
}

NXAtom MatchTypes(const NXAtom *typesToMatch, const NXAtom *orderedTypes)
{
    while (orderedTypes && *orderedTypes) {
	if (IncludesType(typesToMatch, *orderedTypes)) return *orderedTypes;
	orderedTypes++;
    }
    return NULL;
}

NXAtom TextPasteType(const NXAtom *types)
/*
 * Returns the pasteboard type in the passed list of types which is preferred
 * by the Draw program for pasting.  The Draw program prefers PostScript over TIFF.
 */
{
    if (IncludesType(types, NXRTFPboardType)) return NXRTFPboardType;
    if (IncludesType(types, NXAsciiPboardType)) return NXAsciiPboardType;
    return NULL;
}

NXAtom ForeignPasteType(const NXAtom *types)
/*
 * Returns the pasteboard type in the passed list of types which is preferred
 * by the Draw program for pasting.  The Draw program prefers PostScript over TIFF.
 */
{
    NXAtom retval = TextPasteType(types);
    return retval ? retval : MatchTypes(types, [NXImage imagePasteboardTypes]);
}

NXAtom DrawPasteType(const NXAtom *types)
/*
 * Returns the pasteboard type in the passed list of types which is preferred
 * by the Draw program for pasting.  The Draw program prefers its own type
 * of course, then it prefers Text, then something NXImage can handle.
 */
{
    if (IncludesType(types, DrawPboardType)) return DrawPboardType;
    return ForeignPasteType(types);
}

NXAtom *TypesDrawExports(void)
{
    static NXAtom *exportList = NULL;
    if (!exportList) {
	NX_MALLOC(exportList, NXAtom, NUM_TYPES_DRAW_EXPORTS);
	exportList[0] = DrawPboardType;
	exportList[1] = NXPostScriptPboardType;
	exportList[2] = NXTIFFPboardType;
    }
    return exportList;
}



