#ifndef DRAW_FILTER		// OOE 2.0

#import "draw.h"

#import "GraphicView_C_Rubberband.h"

@implementation GraphicView (Rubberband)


- dragSelect:(NXEvent *)event
/*
 * Allows the user the drag out a box to select all objects either
 * intersecting the box, or fully contained within the box (depending
 * on the state of the ALTERNATE key).  After the selection is made,
 * the slist is updated.
 */
{
    int i;
    Graphic *graphic;
    NXPoint p, last, start;
    NXTrackingTimer *timer = NULL;
    NXRect visibleRect, eb, region, oldRegion;
    BOOL mustContain, shift, canScroll, oldRegionSet = NO;

    p = start = event->location;
    [self convertPoint:&start fromView:nil];
    last = start;

    shift = (event->flags & NX_SHIFTMASK) ? YES : NO;
    mustContain = (event->flags & NX_ALTERNATEMASK) ? YES : NO;

    [self lockFocus];

    [self getVisibleRect:&visibleRect];
    canScroll = !NXEqualRect(&visibleRect, &bounds);
    if (canScroll) startTimer(timer);

    PSsetgray(NX_LTGRAY);
    PSsetlinewidth(0.0);

    region.size.width = region.size.height = 0.0;
    event = [NXApp getNextEvent:DRAG_MASK];
    while (event->type != NX_MOUSEUP) {
	if (event->type == NX_TIMER) event->location = p;
	p = event->location;
	[self convertPoint:&p fromView:nil];
	if (p.x != last.x || p.y != last.y) {
	    getRegion(&region, &p, &start);
	    [window disableFlushWindow];
	    if (oldRegionSet) {
		NXInsetRect(&oldRegion, -1.0, -1.0);
		[self drawSelf:&oldRegion :1];
	    }
	    if (canScroll) {
		[self scrollRectToVisible:&region];
		[self scrollPointToVisible:&p];
	    }
	    PSrectstroke(region.origin.x, region.origin.y, 
	    		region.size.width, region.size.height);
	    [self tryToPerform:@selector(updateRulers:) with:(void *)&region];
	    [[window reenableFlushWindow] flushWindow];
	    oldRegion = region; oldRegionSet = YES;
	    last = p;
	    NXPing();
	}
	p = event->location;
	event = [NXApp getNextEvent:DRAG_MASK];
    }

    if (canScroll) stopTimer(timer);

    for (i = [glist count] - 1; i >= 0; i--) {
 	graphic = [glist objectAt:i];
	[graphic getExtendedBounds:&eb];
	
	if (![graphic isLocked] && ![graphic isSelected] && 
	
		#ifdef OOE_SERVER
		[[graphic class] isDragSelectable] &&
		#endif
	
	    ((mustContain && NXContainsRect(&region, &eb)) ||
	     (!mustContain && NXIntersectsRect(&region, &eb)))) {
	    [graphic select];
	}
    }
    [self getSelection];

    if (!dragRect) NX_MALLOC(dragRect, NXRect, 1);
    *dragRect = region;

    NXInsetRect(&region, -1.0, -1.0);
    [self drawSelf:&region :1];
    [self recacheSelection:NO];

    [self tryToPerform:@selector(updateRulers:) with:nil];

    [self unlockFocus];

    return self;
}



@end

#endif
