#ifndef DRAW_FILTER		// OOE 2.0

#import "draw.h"

#import "GraphicView_C_TargetAction.h"

@implementation GraphicView (TargetAction)

/*
 * Target/Action methods to change Graphic parameters from a Control.
 * If the sender is a Matrix, then the selectedRow is used to determine
 * the value to use (for linecap, linearrow, etc.) otherwise, the
 * sender's floatValue or intValue is used (whichever is appropriate).
 * This allows interface builders the flexibility to design different
 * ways of setting those values.
 */

- takeGrayValueFrom:sender
{
    float value;

    value = [sender floatValue];
    [self graphicsPerform:@selector(setGray:) with:&value];
    [window flushWindow];

    return self;
}

- takeLineWidthFrom:sender
{
    id change;
    float width = [sender floatValue];

    change = [[LineWidthGraphicsChange alloc] initGraphicView:self lineWidth:width];
    [change startChange];
	[self graphicsPerform:@selector(setLineWidth:) with:&width];
	[window flushWindow];
    [change endChange];

    return self;
}

- takeLineJoinFrom:sender
{
    int joinValue;
    id change;

    if ([sender respondsTo:@selector(selectedRow)])
        joinValue = [sender selectedRow];
    else
        joinValue = [sender intValue];
    
    change = [[LineJoinGraphicsChange alloc] initGraphicView:self lineJoin:joinValue];
    [change startChange];
	[self graphicsPerform:@selector(setLineJoin:) with:(void *)joinValue];
	[window flushWindow];
    [change endChange];

    return self;
}

- takeLineCapFrom:sender
{
    int capValue;
    id change;

    if ([sender respondsTo:@selector(selectedRow)])
        capValue = [sender selectedRow];
    else
        capValue = [sender intValue];
    
    change = [[LineCapGraphicsChange alloc] initGraphicView:self lineCap:capValue];
    [change startChange];
	[self graphicsPerform:@selector(setLineCap:) with:(void *)capValue];
	[window flushWindow];
    [change endChange];

    return self;
}

- takeLineArrowFrom:sender
{
    int arrowValue;
    id change;

    if ([sender respondsTo:@selector(selectedRow)])
        arrowValue = [sender selectedRow];
    else
        arrowValue = [sender intValue];
    
    change = [[ArrowGraphicsChange alloc] initGraphicView:self lineArrow:arrowValue];
    [change startChange];
	[self graphicsPerform:@selector(setLineArrow:) with:(void *)arrowValue];
	[window flushWindow];
    [change endChange];

    return self;
}

- takeFillValueFrom:sender
{
    int fillValue;
    id change;

    if ([sender respondsTo:@selector(selectedRow)])
        fillValue = [sender selectedRow];
    else
        fillValue = [sender intValue];
    
    change = [[FillGraphicsChange alloc] initGraphicView:self fill:fillValue];
    [change startChange];
	[self graphicsPerform:@selector(setFill:) with:(void *)fillValue];
	[window flushWindow];
    [change endChange];

    return self;
}

- takeFrameValueFrom:sender
{
    if ([sender respondsTo:@selector(selectedRow)]) {
	[self graphicsPerform:@selector(setFramed:) with:(void *)[sender selectedRow]];
    } else {
	[self graphicsPerform:@selector(setFramed:) with:(void *)[sender intValue]];
    }
    [window flushWindow];
    return self;
}

- takeLineColorFrom:sender
{
    id change;
    NXColor color = [sender color];

    change = [[LineColorGraphicsChange alloc] initGraphicView:self color:&color];
    [change startChange];
	[self graphicsPerform:@selector(setLineColor:) with:&color];
	[window flushWindow];
    [change endChange];

    return self;
}

- takeFillColorFrom:sender
{
    id change;
    NXColor color = [sender color];

    change = [[FillGraphicsChange alloc] initGraphicView:self];
    [change startChange];
	[self graphicsPerform:@selector(setFillColor:) with:&color];
	[window flushWindow];
    [change endChange];

    return self;
}

- takeFormEntryStatusFrom:sender
{
    [self graphicsPerform:@selector(setFormEntry:) with:(void *)[sender intValue]];
    [window flushWindow];
    return self;
}


@end

#endif


