#ifndef DRAW_FILTER		// OOE 2.0

#import "draw.h"

#import "GraphicView_C_TargetActionCmds.h"

@implementation GraphicView (TargetActionCmds)

/*
 * Target/Action methods.
 */

- delete:sender
{
    int i;
    Graphic *graphic;
    id change;
    
    #ifdef OOE_SERVER
    if([self isClipRectSelected]) {
    	NXBeep();
	return self;
    }
    #endif

    i = [slist count];
    if (i > 0) {
	change = [[DeleteGraphicsChange alloc] initGraphicView:self];
	[change startChange];
	    [self graphicsPerform:@selector(deactivate)];
	    [slist makeObjectsPerform:@selector(activate)];
	    while (i--) {
		graphic = [slist objectAt:i];
		[glist removeObject:graphic];
		[graphic wasRemovedFrom:self];
	    }
	    if (originalPaste == [slist objectAt:0]) [slist removeObjectAt:0];
	    [slist free];
	    slist = [[List allocFromZone:[self zone]] init];
	    gvFlags.groupInSlist = NO;
	    [window flushWindow];
        [change endChange];
	return self;
    } else {
        return nil;
    }
}

- selectAll:sender
/*
 * Selects all the items in the glist.
 */
{
    int i;
    Graphic *g;
    NXRect visibleRect;

    i = [glist count];
    if (!i) return self;

    [slist free];
    slist = [[List allocFromZone:[self zone]] init];
    [[cacheWindow contentView] lockFocus];
    while (i--) {
	g = [glist objectAt:i];
	
	if (![g isLocked]
			
			) {
	    [g select];
	    [g draw:NULL];
	    [slist insertObject:g at:0];
	    gvFlags.groupInSlist = gvFlags.groupInSlist || 
	    		[g isKindOf:[Group class]];
	}
    }
    
    [Graphic showFastKnobFills];
    [[cacheWindow contentView] unlockFocus];
    [self getVisibleRect:&visibleRect];
    if ([self canDraw]) {
	[self lockFocus];
	[self drawSelf:&visibleRect :1];
	[self unlockFocus];
    }
    if (sender != self) [window flushWindow];
    gvFlags.selectAll = YES;
    
    return self;
}

- deselectAll:sender
/*
 * Deselects all the items in the slist.
 */
{
    NXRect sbounds;

    if ([slist count] > 0) {
	[self getBBox:&sbounds of:slist];
	[slist makeObjectsPerform:@selector(deselect)];
	[self cache:&sbounds andUpdateLinks:NO];
	[slist free];
	slist = [[List allocFromZone:[self zone]] init];
	gvFlags.groupInSlist = NO;
	if (sender != self) [window flushWindow];
    }

    return self;
}

- changeAspectRatio:sender
{
    id change;
    
    change = [[AspectRatioGraphicsChange alloc] initGraphicView:self];
    [change startChange];
	[self graphicsPerform:@selector(sizeToNaturalAspectRatio)];
    [change endChange];

    [window flushWindow];
    return self;
}

- showLinks:sender
/*
 * If the tag of the sender is non-zero, then linked items are
 * shown with a border around them (see redrawLinkOutlines: in gvLinks.m).
 */
{
    [linkManager setLinkOutlinesVisible:[sender selectedTag] ? YES : NO];
    return self;
}


/* The venerable print: method. */

- print:sender
{
    return [self printPSCode:sender];
}

- changeFont:sender
{
    id change;

    if ([window firstResponder] == self) {
	change = [[MultipleChange alloc] initChangeName:FONT_OPERATION];
	[change startChange];
	    [self graphicsPerform:@selector(changeFont:) with:sender];
	    [window flushWindow];
	[change endChange];
    }
    return self;
}




@end

#endif


