//
//	Xanthus
//	Copyright (c) 1992-95 Bjorn Backlund.  
//	All rights reserved. 
//

#import "filter_all.h"

#import "FilterApp_C_Service.h"

@implementation FilterApp (Service)

- (int)draw2eps:pb
    userData:(const char *)userData
    error:(char **)msg
{
	char *data = NULL; 
	int length = 0;
	const char *filetype = "NXTypedFilenamePboardType:draw";
	const char *type = NULL;
	
    	if(FilterDebug)
		NXLogError("Filter: draw2eps -> %s", [pb name]);
	
	type = [pb findAvailableTypeFrom: &filetype num:1];
	
    // not ok?
    
	if(type == NULL) {
		*msg = "No NXTypedFilenamePboardType:draw in pboard";
		NXLogError(*msg);
		return 1;
	}
	
    // get filename
    
	[pb readType:filetype 
		data: &data 
		length: &length];
			
	if(FilterDebug) NXLogError("Filter: file = %s.", data);
		
    //	do the actual conversion
		
	strcpy(theFile, data);
	[self convertDrawFileAsEPSTo:pb];
    	
    // free filename
    
	[pb deallocatePasteboardData: data
		length: length];
    
	return 0;
}

- convertDrawFileAsEPSTo:pb;
{	
    	NXAtom rettypes[8];
	int k = 0;
	NXStream *stream = NULL;
	
    // produce eps
    	
	stream = NXOpenMemory(NULL, 0, NX_READWRITE);
	
	[self convertDrawFileToEPSStream:stream];
	
    // output eps to pboard
    
	rettypes[k++] = NXPostScriptPboardType; 
	[pb declareTypes:rettypes 
		num:k 
		owner:self];
	[pb writeType:NXPostScriptPboardType 
		fromStream:stream];	
    
    // exit
    
	if(stream) NXCloseMemory(stream, NX_FREEBUFFER);
	
	return self;
}

- convertDrawFileToEPSStream: (NXStream *)stream;
{	
	
    // open doc
    
	if([self openDrawFile] == nil) return nil;
	
    // produce eps
    	
	[theDocument copyAsEPSTo:stream];
	NXSeek(stream, 0, NX_FROMSTART);
		
    // exit
    
	[theDocument close];
	theDocument = nil;
	
	return self;
}

- convertDrawFileToEPS;
{	
	NXStream *stream = NULL;
	
    // produce eps
    	
	stream = NXOpenMemory(NULL, 0, NX_READWRITE);
	
	[self convertDrawFileToEPSStream:stream];
	
    // output eps to stdout
    
	NXSeek(stream, 0, NX_FROMSTART);
	while(!NXAtEOS(stream)) {
	
		int c = NXGetc(stream);
		
		if(c != EOF) putchar(c);
	}
		
    // exit
    
	if(stream)
		NXCloseMemory(stream, NX_FREEBUFFER);
	
	return self;
}


@end


