@interface TextGraphic : Graphic
{
    char *data;			/* the rich text */
    int length;			/* the length of data */
    Font *font;
    View *editView;
    GraphicView *graphicView;
    Text *fe;			/* the field editor text object      */
    				/* used for editing between edit:in: */
				/* and textDidEnd:endChar:           */
    NXDataLink *link;
    char *name;
}

/* Factory methods overridden from superclass */

+ (BOOL)isEditable;
+ cursor;

/* Initialization method */

#ifndef DRAW_FILTER		// OOE 2.0
+ (BOOL)canInitFromPasteboard:(Pasteboard *)pboard;
#endif

- init;
- finishedWithInit;

- initEmpty;

- initFromStream:(NXStream *)stream;
- initFromPasteboard:(Pasteboard *)pboard;

#ifndef DRAW_FILTER		// OOE 2.0
- (NXRect)reinitFromPasteboard:(Pasteboard *)pboard;
- (NXRect)reinitFromStream:(NXStream *)stream;
#endif


- initFormEntry:(const char *)name localizable:(BOOL)isLocalizable;



/* Link methods */

#ifndef DRAW_FILTER		// OOE 2.0
- setLink:(NXDataLink *)aLink;
#endif

- (NXDataLink *)link;



/* Instance methods overridden from superclass */

- (const char *)title;

#ifndef DRAW_FILTER		// OOE 2.0
- (BOOL)create:(NXEvent *)event in:(GraphicView *)view;
- (BOOL)edit:(NXEvent *)event in:(View *)view;
- performTextMethod:(SEL)aSelector with:(void *)anArgument;
#endif

- draw;
#ifndef DRAW_FILTER		// OOE 2.0
- changeFont:sender;
#endif

- (Font *)font;
- (BOOL)isOpaque;
- (BOOL)isValid;
- (BOOL)isFormEntry;
- setFormEntry:(int)flag;
- (BOOL)writeFormEntryToStream:(NXStream *)stream;
- (NXColor)lineColor;
- (NXColor)fillColor;
- (NXCoord)baseline;
- moveBaselineTo:(NXCoord *)y;




/* Public methods */

#ifndef DRAW_FILTER		// OOE 2.0
- prepareFieldEditor;
- resignFieldEditor;
#endif

- (BOOL)isEmpty;
- (char *)data;
- setData:(char *)newData;
- (int)length;
- setLength:(int)newLength;
- setFont:newFont;




/* Text delegate methods */

#ifndef DRAW_FILTER		// OOE 2.0
- textDidResize:textObject 
	oldBounds:(const NXRect *)oldBounds 
	invalid:(NXRect *)invalidRect;
- textDidEnd:textObject 
	endChar:(unsigned short)endChar;
#endif




/* Archiving methods */

- awake;
- read:(NXTypedStream *)stream;
- write:(NXTypedStream *)stream;

@end
