//
//	Xanthus
//	Copyright (c) 1992, 1993, 1994 Bjorn Backlund & Jesper Lundh.  
//	All rights reserved. 
//


#import "app.h"


#define INTERFACE_PBOARDTYPE		"Yap++_OOE_InterfaceInfo"
 
//
//
//

#define	OOE_DEFAULT_PS 		"%!PS-Adobe-2.0\n%%BoundingBox: 0 0 100 100\n"

//
//
// 

#define YAP_BBOX		"%%BoundingBox:"
#define YAP_BBOX_LEN		14
#define YAP_TEST_LEN		64

int OOE_findBoundingBox(NXRect *aRect, const char *text)
{
	const char *start;
	char buf[YAP_TEST_LEN];
	int i;
	
	start = strstr(text, YAP_BBOX);
	if(start == NULL) return 0;
	
	start += YAP_BBOX_LEN;
	
	while(*start == ' ') start++;
	
	for(i=0; i<YAP_TEST_LEN-1; i++) {

		if(*start == '\n') break;

		buf[i] = *start;
		start++;

	}
	buf[i] = '\0';

	sscanf(buf, "%f %f %f %f", 
		&aRect->origin.x,
		&aRect->origin.y,
		&aRect->size.width,
		&aRect->size.height);
		
	return 1;
}







@implementation YapDocument (OOE_Server)

- window;
{
	return [document window];
}

- setTitle_OOE:sender;
{
	char buf[MAXPATHLEN+1];
	
	if(theOOEClientProxy == nil && isOOEServer) {
	
		strcpy(buf, "Embedded: **** DEAD EDITOR ****");
		
	} else if(theOOEClientPath == NULL) {
	
		strcpy(buf, "Embedded: UNTITLED");
	} else { 
	
		sprintf(buf, "Embedded: %s", theOOEClientPath);
	}
	
	[[self window] setTitle:buf];
	
	return self;
}

- docWillClose_OOE:sender;
{
	if(theOOEClientProxy == nil) return self;
	
	if(!isOOEServer) return self;
	
	if(OOE_Is_Debug)
		NXLogError("OOE, DOC, docWillClose_OOE");

	[[theOOEClientProxy connectionForProxy] 
		unregisterForInvalidationNotification:self];
	
	NX_DURING
		[theOOEClientProxy OOE_documentWillClose:self];	
	NX_HANDLER
		NXLogError("DOC %s, OOE: Error %d for "
				"'OOE_documentWillClose:'",
				[self name],
				NXLocalHandler.code);
	NX_ENDHANDLER
	
	[NXConnection removeObject:self];

	return self;
}

- free_OOE;
{
	if(OOE_Is_Debug)
		NXLogError("OOE, DOC, free_OOE");

	NX_DURING
	
		//[theOOEClientProxy free];	// 940824
						// causes timeout
						// why?
		theOOEClientProxy = nil;
		
	NX_HANDLER
		
		if(OOE_Is_Debug)
			NXLogError("ERROR, DOC, OOE: Error %d when "
					"freeing theOOEClientProxy.",
					NXLocalHandler.code);
			
	NX_ENDHANDLER
	
	if(theOOEClientPath != NULL) {
		NX_FREE(theOOEClientPath);
		theOOEClientPath=NULL;
	}
	
	return self;
}

- clipRect;
{
	return [document docView];
}

- initForNewOOEClient:client;
{
    // set up default postscript
	
	[[document docView] setText: OOE_DEFAULT_PS];
	
    // connect to the OOE client
    
	[self OOE_connectToClient:client];
	
    // bring window to front
    
    	[[self window] makeKeyAndOrderFront:self];
	 
	return self;
}
 
- initForOldOOEClient:client;
{
    // connect to client
    
	[self OOE_connectToClient:client];
	
    // get data from pasteboard
    
	[self OOE_retrieveDataFromPasteboard];
	
    // get interface data from pasteboard
    
	[self OOE_retrieveInterfaceInfoFromPasteboard];
	
    // bring window to front
    
    	[[self window] makeKeyAndOrderFront:self];
	
	return self;
}
 
- (int)isOOEServer;
{
	return isOOEServer;
}

- (int)OOE_connectToClient:client;
{
	id connectionToClient;
	const char *path = NULL;
	 
	if(OOE_Is_Debug) 
		NXLogError("DOC %s: OOE_connectToClient: = %s", 
				[self name],
				[client name]);

    // remember that I'm an ooe document

	isOOEServer = YES;

    // store pointer to client object

	theOOEClientProxy = client;

    // tell do system that I want to know when the client dies

    	connectionToClient = [client connectionForProxy];
    	[connectionToClient registerForInvalidationNotification:self];
      	  
    	[client setProtocolForProxy:@protocol(OOE_ClientInterface)];
	
    // obtain path for client document
    
	NX_DURING
		path = [client OOE_documentPath:self];
	NX_HANDLER
		NXLogError("DOC %s, OOE: Error %d for 'OOE_documentPath:'",
				[self name],
				NXLocalHandler.code);
	NX_ENDHANDLER
	
    // update window title

	[self OOE_document: client savedAs:path];
	
	return 1;
}

- senderIsInvalid:sender;
{
	if(theOOEClientProxy == nil) {
		if(OOE_Is_Debug)
			NXLogError("OOE, DOC (Non Server) %s, INVALID: %s", 
				[self name],
				[sender name]);
		return self;
	}
	
	if(OOE_Is_Debug)
		NXLogError("OOE, DOC %s: INVALID: %s", 
			[self name],
			[sender name]);
	

	theOOEClientProxy = nil;
	
	[self setTitle_OOE:self];
	
    	return self;
}

//
//
//

- getBoundingBox: (NXRect *)aRect;
{
	NXStream *stream = NXOpenMemory(NULL, 0, NX_WRITEONLY);
	char *buf;
	int len, dummy;
	int a;
	
	[[document docView] writeText: stream];
	NXGetMemoryBuffer(stream, &buf, &len, &dummy);
	
	a = OOE_findBoundingBox(aRect, buf);
	
	NXCloseMemory(stream, NX_FREEBUFFER);
	
	if(a) return self;
	
	return nil;
}

//
//
//

- updateOOETarget:sender;
{
	if(OOE_Is_Debug) 	
		NXLogError("DOC %s: updateOOETarget:", [self name]);
	
	[theOOEClientProxy OOE_documentWasChanged:self];
	[theOOEClientProxy OOE_orderClientDocumentFront:self];

	return self;
}

- orderClientDocumentFront:sender;	// OOE 2
{
	if(OOE_Is_Debug) NXLogError("DOC: orderClientDocumentFront:");
	
	[theOOEClientProxy OOE_orderClientDocumentFront:self];

	return self;
}

//
// OOE_DocumentInterface
//

- (int)OOE_sizeTo:(NXCoord)w:(NXCoord)h;
{
	
	if(OOE_Is_Debug) NXLogError("DOC %s: OOE_sizeTo:",  [self name]);

	// NOT YET USED BY OOE PROTOCOL
	
	return 1;
}

- (int)OOE_getSize:(NXSize *)aSize;
{
	NXRect aRect = {0,0,300,300};	
	
	if(OOE_Is_Debug) NXLogError("DOC %s: OOE_getSize:", 
					[self name]);

	if([self getBoundingBox: &aRect] != nil) {
		*aSize = aRect.size;
		return 1;
	}
	
	return 0;
}

- (int)OOE_orderDocumentFront:sender;
{
	if(OOE_Is_Debug) NXLogError("DOC %s: OOE_orderDocumentFront:",
					[self name]);

	[NXApp activateSelf:YES];
	[[self window] makeKeyAndOrderFront:nil];
	
	return 1;
}

- (void)OOE_closeDocument:sender;
{
	if(OOE_Is_Debug) 
		NXLogError("DOC %s: OOE_closeDocument:",
			[self name]);

	isOOEServer = 0;

	[[self window] setDocEdited:0];
	[self closeDocument:"FooBar" andWindow:YES];
	
	return; 
}

- (int)OOE_document: client savedAs:(const char *)path;
{
	if(path == NULL) path = "UNTITLED";
	
	if(theOOEClientPath != NULL)
		NX_FREE(theOOEClientPath);
	theOOEClientPath = NXCopyStringBuffer(path);
	
	[self setName: theOOEClientPath];
	
	return 1;
}

- (int)OOE_isDocEdited;
{
	if(OOE_Is_Debug) 
		NXLogError("DOC %s: OOE_isDocEdited", [self name]);

	return [[self window] isDocEdited]; 
}

//
// OOE Data Transfer
//

- OOE_retrieveDataFromPasteboard;
{
	id pboard;
	NXStream *stream = NULL;
	const NXAtom *types;
	NXAtom psType = NXPostScriptPboardType;
	
 	if(OOE_Is_Debug) 
		NXLogError("DOC: OOE_retrieveDataFromPasteboard:");

	pboard = [Pasteboard newName: OOE_PASTEBOARD];
	if(pboard == nil) {
	
		NXLogError("ERROR: No '%s' pasteboard!",
					NXPostScriptPboardType);
		return nil;
	}
	
	types = [pboard types];
	
	if(OOE_Is_Debug) {
	
		NXLogError("OOE: Change count for %s is %d:",	
				[pboard name], 
				[pboard changeCount]);
		NXLogError("OOE: Types on the pboard %s are:",	
				[pboard name]);
				
		while(types[0]) {
			NXLogError("\t %s",types[0]);	
			types++;
		}
	}

	if([pboard findAvailableTypeFrom: &psType  
		num:1] == NULL) {
	
		if(OOE_Is_Debug)
			NXLogError("NOTE: No '%s' type in pasteboard '%s'!",
				NXPostScriptPboardType, 
				OOE_PASTEBOARD);
	
		goto defText;
	};

	stream = [pboard readTypeToStream: NXPostScriptPboardType];
	if(stream == NULL) {
	
		NXLogError("ERROR: No '%s' type in pasteboard '%s'!",
				NXPostScriptPboardType, 
				OOE_PASTEBOARD);
		return nil;
	}
	
 	if(OOE_Is_Debug) 
		NXLogError("\t reading text stream...");

    // HANDLERS NEW 941107
    
	NX_DURING
		[[document docView] readText:stream];
	NX_HANDLER
		NXLogError("ERROR, OOE: Exception %d when reading text stream",
				NXLocalHandler.code);
	NX_ENDHANDLER
	
	if(stream)
		NXCloseMemory(stream, NX_FREEBUFFER);	// 941113
	
    // NEW 941107
    
    defText:
    
	if([[document docView] charLength] == 0) {
	
		NXLogError("No PostScript code. Adding default...");
		
		[[document docView] setText: OOE_DEFAULT_PS];
	}
	
    // END NEW
    
	[[document docView] sizeToFit];
	
	return self;
}


- OOE_retrieveInterfaceInfoFromPasteboard;
{
	id pboard;
	NXStream *stream = NULL;
	const char *type = INTERFACE_PBOARDTYPE;
	id retval = nil;
	
 	if(OOE_Is_Debug) 
		NXLogError("DOC: OOE_retrieveInterfaceInfoFromPasteboard: %s",
				type);

    // get ooe pasteboard
    
	pboard = [Pasteboard newName: OOE_PASTEBOARD];
	if(pboard == nil) {
	
		NXLogError("ERROR: No '%s' pasteboard!",
				OOE_PASTEBOARD);
		return nil;
	}
	
    // check if client has provided us with interface info
    
	[pboard types];

	if([pboard findAvailableTypeFrom: &type  
		num:1] == NULL) {
		
		if(OOE_Is_Debug) 
			NXLogError("OOE CLIENT %d: "
				"No interface data in pasteboard", self);
		return self;
	};
		
    // read the info
    
	stream = [pboard readTypeToStream: type];
	if(stream == NULL) {
	
		NXLogError("ERROR: No '%s' type in pasteboard '%s'!",
				type, OOE_PASTEBOARD);
		return nil;
	}
	
	retval = [self readOOEInterfaceInfoFrom:stream];
	
	if(stream)
		NXCloseMemory(stream, NX_FREEBUFFER);	// 941113
	
	return retval;
}

- promptOldOOEVersion;
{
	[NXApp reportOOEProblem: "OOE Version Problem"
	
		format: "The OOE object has been created with "
			"an older version of %s "
			"and can't be opened!",
		
			[NXApp appName]];
	
	return self;
}

- promptNewOOEVersion;
{
	[NXApp reportOOEProblem: "OOE Version Problem"
	
		format: "The OOE object has been created "
			"with a newer version of %s "
			"and can't be opened!",
		
			[NXApp appName]];
	
	return self;
}


- readOOEInterfaceInfoFrom:(NXStream *)stream;				
{
	NXTypedStream *ts = NULL;
	int diskOOEVersion = 0;
	id retval = self;
	NXRect windowFrame;
	
 	if(OOE_Is_Debug) 
		NXLogError("DOC: readOOEInterfaceInfoFrom:");

    // open a typed stream
    
	ts = NXOpenTypedStream(stream, NX_READONLY);
	if(ts == NULL) {
		NXLogError("ERROR, DOC, OOE, readOOEInterfaceInfoFrom"
				"Could not open typed stream!");
		return nil;
	}
	
	NXSetTypedStreamZone(ts, [self zone]);
    	
    // read ooe version
    
    	NX_DURING
		NXReadType(ts, "i", &diskOOEVersion);
    	NX_HANDLER
		retval = nil;
    	NX_ENDHANDLER
	
    // check versions
    
	if(diskOOEVersion < OOE_Version_CanOpen) {
	   
		[self promptOldOOEVersion];
		NXCloseTypedStream(ts);
		return nil;
		
	}

	if((diskOOEVersion > OOE_Version) > 0) {
	    
		[self promptNewOOEVersion];
		NXCloseTypedStream(ts);
		return nil;
		
	}

    // read interface info
    
    	NX_DURING
	
    		NXReadRect(ts, &windowFrame);
		
	NX_HANDLER
		NXLogError("ERROR, DOC: Error %d for "
				"readOOEInterfaceInfoFrom:",
				NXLocalHandler.code);
	    	retval = nil;
	NX_ENDHANDLER
	
    // update window
	
	[[self window] placeWindowAndDisplay: &windowFrame];
	
	return retval;
}

//
//
//


- (int)OOE_provideData:sender;
{
	NXStream *epsStream = NULL;
	NXStream *docStream = NULL;
	id pboard = nil;
	const char *types[3];
	int k = 0;
	
	pboard = [Pasteboard newName: OOE_PASTEBOARD];
	
 	if(OOE_Is_Debug) 
		NXLogError("DOC %s: OOE_provideData:", [self name]);

    // create document data = eps
    
   	epsStream = NXOpenMemory(NULL, 0, NX_READWRITE);
    	[self copyAsEPSTo: epsStream];
	NXSeek(epsStream, 0, NX_FROMSTART);
	types[k++] = NXPostScriptPboardType;
	
    // create doc interface info
	
    	docStream = NXOpenMemory(NULL, 0, NX_WRITEONLY);
    	[self copyAsOOEInterfaceInfoTo: docStream];
	NXSeek(docStream, 0, NX_FROMSTART);
	types[k++] = INTERFACE_PBOARDTYPE;
	
    // pboard setup
    
	[pboard declareTypes: types 
		num: k 
		owner: NXApp];	
	
    // write eps
    
	[pboard writeType: NXPostScriptPboardType
		fromStream:epsStream];

    // write doc data
    
	[pboard writeType: INTERFACE_PBOARDTYPE
		fromStream:docStream];
		
    // cleanup
    
	if(epsStream != NULL) NXCloseMemory(epsStream, NX_FREEBUFFER);
	if(docStream != NULL)NXCloseMemory(docStream, NX_FREEBUFFER);

	[[self window] setDocEdited:NO];
	
 	if(OOE_Is_Debug) 
		NXLogError("DOC %s: EXIT OOE_provideData:",
				 [self name]);
	
	return 1;
}

- copyAsEPSTo:(NXStream *)stream;				
{
 	if(OOE_Is_Debug) 
		NXLogError("DOC %s: copyAsEPSTo:", 
				[self name]);

	[[document docView] writeText: stream];
	
	return self;
}

- copyAsOOEInterfaceInfoTo:(NXStream *)stream;				
{
	NXTypedStream *ts;
	int ooeVersion = OOE_Version;
	NXRect aRect;
	
	[[self window] getFrame: &aRect];
	
 	if(OOE_Is_Debug) 
		NXLogError("DOC %s: copyAsOOEInterfaceInfoTo:", [self name]);

	ts = NXOpenTypedStream(stream, NX_WRITEONLY);
	if(ts == NULL) {
		NXLogError("ERROR, DOC, OOE, copyAsOOEInterfaceInfoTo"
				"Could not open typed stream!");
		return nil;
	}
	
	
 	NX_DURING

   // write version
	
	NXWriteType(ts, "i", &ooeVersion);		// ooe version

    // write data 
    
	NXWriteRect(ts, &aRect);
	
	NX_HANDLER
		NXLogError("ERROR, DOC %s: Error %d for write data "
				"in copyAsOOEInterfaceInfoTo:",
				NXLocalHandler.code);
	NX_ENDHANDLER
	
    // close typed stream
    
	NX_DURING
		NXCloseTypedStream(ts);
	NX_HANDLER
		NXLogError("ERROR, DOC %s: Error %d for NXCloseTypedStream()"
			"in copyAsOOEInterfaceInfoTo:",
			NXLocalHandler.code);
	NX_ENDHANDLER
	
 	if(OOE_Is_Debug) 
		NXLogError("DOC %s: EXIT copyAsOOEInterfaceInfoTo:",
				[self name]);
	
	return self;
}

- copyAsTIFFTo:(NXStream *)stream;	// TO BE USED
{
 	if(OOE_Is_Debug) 
		NXLogError("DOC %s: copyAsTIFFTo:", [self name]);

	return self;  
}

//
//
//

- (int)checkNotOOEDocument:(const char *)title;		// TO BE USED
{
	if(isOOEServer) {
	
		NXRunAlertPanel(title,
		
			"This is an OOE document.",
			
			"OK",
			NULL,
			NULL);
		return NO;
	}

	return YES;
}

@end


